/* 
 *  DiskGraf  Copyright (C) Roy Longbottom 2004
 * 
 *        Version 1.3A 
 *
 *  What the benchmark does
 *
 *  The program measures disk write and read speeds at different
 *  block sizes, producing graphs of megabytes per second and CPU
 *  utilisation. When the program starts a few measurements are
 *  made and warnings given if DMA does not appear to be set,
 *  writing is much slower than reading or speed is generally slow.
 *  In the latter case, default parameters of 5 files, each 8 MB,
 *  with block sizes between 1 KB and 1024 KB will be reduced to
 *  ensure that running time is not too long. Up to 16 files of
 *  10 GB can be chosen which can be used to demonstrate performance
 *  variations over the whole surface to 160 GB capacity. The
 *  program checks that space is available. Speed and CPU
 *  utilisation is shown separately for each file. This can
 *  show that the disk is fragmented or other peculiarities.
 * 
 *  Data files are written in the folder containing the exe file.
 *  Results are saved in text file DiskGraf.txt, a menu option
 *  being available to load and redisplay. Optionally, the graph
 *  can be saved in BMP format. These files will also be in the
 *  same folder. The BMP graph is 135 KB but can be loaded with
 *  appropriate software and saved as a compressed GIF file to
 *  shrink size to around 16 KB. 
 *
 *  Modules
 * 
 *  config.cpp - via WhatConfig() identifies CPU type/speed, memory
 *               size, OS version, also running times via getStartTime()
 *               and getRunSecs() and date/clock time via local_time()
 *
 *  diskstats.cpp - Via GetDiskStats() to get drive free space and
 *               total capacity for Win95 or later OSs
 *
 *  DiskGraf.rc - menus and and dialog boxes
 *    MENU_HELP and MENU_ABOUT -> AboutDlgProc()
 *    MENU_PARAM -> RunOptions() as PARAM_GRAPH Variables button
 *    MENU_RUNGRAPH as RUN_BUTTON
 *    MENU_QUICKTEST as QUICK_RUN button
 *    MENU_LOAD uses GetFileName() and loadFile() to load/display
 *              graph from any log file
 *    MENU_SAVE uses GetFileName() and saveBmp() to save graph
 *              as a BMP file
 *    MENU_VIEWGRAPH to re-display main graph
 *    MENU_DMAGRAPH to display DMA graph
 *    MENU_RANDGRAPH to display random access graph
 *    MENU_EXIT as Exit button FINISH_GRAPH
 *
 *  DiskGraf.h and DiskGraf.ico
 *
 *  RunOptions() - This has Comboboxes for MINBLOCKSIZE and
 *  MAXBLOCKSIZE updated by UpdateCombos(), Edit boxes GRAPH_MB
 *  (File Size). NUMBERFILES, GRAPH_FILE (bmp file name), NOTE_1,
 *  NOTE_2, NOTE_3 (user info), Radio Buttons USE_CACHE, MEASURE_UT,
 *  and BIG_RAM also using UpdateRadioButtons()
 *
 *  DiskGraf.cpp
 *
 *  Particularly with Windows XP, on running a benchmark that displays
 *  on-going results, the window for the latter can be covered by
 *  another window. To overcome this, all tests are run via the WinMain 
 *  message processing loop using a phase variable:
 * 
 *  phase 1 - initial assignments via startTests()
 *  phase 2 - write1File()/writeGFiles for each file then
              read1File()/readGFiles for each file
 *  phase 3 - read1DMA()/readDMA for each block size then initRand()
 *  phase 4 - read1Rand()/readRAND() for each required part file size,
              eraseFiles()
 *  phase 5 - endTests()
 *
 *  The main window is used to display all information and graphs.
 *  WindowProc() creates 4 buttons. RUN_BUTTON and QUICK_RUN set
 *  phase = 1, PARAM_GRAPH executes RunOptions() and FINISH_GRAPH is
 *  exit. The graphs are drawn by drawGraphs() via reView() which is
 *  used after each file read/write or when anything has changed.
 * 
 *  Functions
 *
 *  WinMain() - usual Windows functions + testFirst() and runs tests
 *  InitApp() - usual Windows functions, processes command line,
 *              allocates variable values
 *  WindowProc() - usual Windows functions for msgs, buttons & menus
 *  AboutDlgProc() - displays Help and About
 *  RunOptions() - displays variable options dialog box
 *  UpdateRadioButtons() - updates RunOptions() controls
 *  UpdateCombos() - updates RunOptions() controls
 *  CleanUpAndPostQuit() - delete and free objects
 *  testFirst() - before window being available, writes short files
 *                with small block sizes then runs quickRun(). These
 *                are to adjust parameters for slow devices and to
 *                provide warning/comment messages.
 *  startTests() - allocate memory, create data
 *  write1File() - set parameters, run writeGFiles(), deal with errors
 *  writeGFiles() - check free space and write a file measuring time 
 *  read1File() - run readGFiles(), deal with errors
 *  readGFiles() - read a file measuring time 
 *  read1DMA() - run readDMA(), deal with errors
 *  readDMA() - re-read same block up to 100 times, measure time
 *  initRand() - calculate variables
 *  read1Rand() - run readRAND(), deal with errors
 *  readRAND() - read 100 random blocks
 *  quickRun() - write/read 2 files 
 *  eraseFiles()
 *  drawGraphs() - display text and graphs
 *  reView() - run drawGraphs()
 *  saveResults() - to log file
 *  saveErrors() - msgs to log file 
 *  saveBmp() - save copy of screen to .bmp file
 *  loadFile() - load selected log file and displays graphs
 *  GetFileName() - used for saveBmp() and loadFile() 
 *  endTests() - free memory and saveResults()
 *  StartThread() - in WinMain() to create a second thread used
 *                 to execute readme() at low priority, soaking
 *                 free CPU time to measure CPU utilisation.
 *                 This is started then suspended.
 *  readme() - assembly code with simple add instructions
 *  CalibrateTimer() - executes readme() to obtain countsPerSec with no I/O
 *  CountLoop() - executes CountThread() contiuously when the
 *                thread is resumed
 *  CountThread() - executes readme() and counts the number of accesses
 *
 *  Procedure for each CPU utilisation measurement is:
 *
 *  CalibrateTimer();
 *  threadCount = 0;
 *  ResumeThread(hThreadHandle);    
 *  SetThreadPriority(hThreadHandle, THREAD_PRIORITY_IDLE);
 *
 *  Write or read I/O
 *
 *  SuspendThread(hThreadHandle);
 *
*/

 
#define NAME "Test"
#define TITLE " Roy Longbottom's Disk Graph Benchmark "
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <direct.h>
#include <commdlg.h>
#include "diskgraf.h"

//  ************** PROTOPTYPES *****************

static BOOL InitApp(HINSTANCE, LPSTR, int);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);

BOOL  FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
BOOL  FAR PASCAL RunOptions( HWND, UINT, WPARAM, LPARAM);
BOOL  FAR PASCAL UpdateRadioButtons( HWND hdlg);
BOOL  FAR PASCAL UpdateCombos( HWND, int, int);

BOOL  writeGFiles(int, int);
BOOL  readGFiles(int, int);
BOOL  readDMA();
BOOL  readRAND();
BOOL  write1File();
BOOL  read1File();
BOOL  read1DMA();
BOOL  read1Rand();
BOOL  reView(int, int);
BOOL  drawGraphs(int, int, HDC);
BOOL  quickRun();

BOOL  saveResults();

BOOL  StartThread();
DWORD WINAPI CountLoop(LPVOID inParam);
BOOL  WINAPI CountThread();
BOOL  CalibrateTimer();
BOOL  saveErrors();
BOOL  saveBmp();
BOOL  testFirst();
BOOL  loadFile();

BOOL  endTests();
BOOL  startTests();
BOOL  eraseFiles();
BOOL  initRand();

static BOOL GetFileName();
void readme();
 

// ************* GLOBAL VARIABLES **********

int *myData;

HANDLE hThreadHandle;
LPVOID inputParam;
int  threadCount = 0;
int  threadPass = 1;
BOOL  goThread;
double countsPerSec;
char    title[120];
char    title2[120];
char    title3[120];
char    versionIn[50] = " ";
char    versionIs[50] = " ";

BOOL bQuit = FALSE;
BOOL    commandError = FALSE;

HWND   mainHWND;

HINSTANCE  MyInstance;
RECT    clientRect;

int     gBlockSize[11];
int     minBlockChoice;
int     blksizes = 11;
int     maxBlockChoice;
int     totalFiles = 5;
int     maxfiles = 16;
int     numberBlocks;
int     maxMbytes = 10240;
int     wordsPerBlock;
DWORD   grafmbytes = 8;
DWORD   bytesPerBlock;
DWORD   bytesPerfile;
DWORD   maxBytesPerBlock = 1048576;
DWORD   arrayBytes;
DWORD   arrayWords;
int     bigOne;

BOOL   useCache = FALSE;
BOOL   measureCPU = TRUE;
BOOL   largeRAM = FALSE;

double cpuUt[1000];

int    total = 0;
char   currentDir[MAX_PATH];
char   diskPart[30];

char   fileNames[25][512];
char   fileMessage[256];
char   errorMsg[500];

BOOL    autoRun = FALSE;

char    logfile[256] = "DiskGraf.txt";
char    bmpfile[256] = "DiskGraf.bmp";

char    windowText[256];
char    commandLine[1024] = "";
double  maxCount = 0;

int  screenWide = 640;
int  screenHigh = 480;
double GWriteMBPS1[11][20];
double GReadMBPS1[11][20];
double GWriteUt1[11][20];
double GReadUt1[11][20];
double GWriteMBPS2[11][20];
double GReadMBPS2[11][20];
double GWriteUt2[11][20];
double GReadUt2[11][20];
char  note1[100] = "";
char  note2[100] = "";
char  note3[100] = "";
BOOL   grafSaved = FALSE;
BOOL   readWrite = FALSE;
RECT   recwin;
int    xtraWide; 
short  keyPress;
char   timeday1[30];
BOOL    testStarted = FALSE;
char   grafFile[256];
char   fileToRead[256];
BOOL    displayOnly = FALSE;
WIN32_FIND_DATA W32DiskData;
int   butT = 280;
int   butL = 520;
int   butW = 80;
int   butH = 22;
BOOL  loadIt;
BOOL  fromLog = FALSE;
BOOL  useDMA = FALSE;
BOOL  viewDMA = FALSE;
double  DMAcpuUt[11];
double  DMAMBPS[11];
BOOL  useRand = FALSE;
BOOL  viewRand = FALSE;
double  RandcpuUt[11];
double  RandMsecs[11];
double  randomNumber[1000];

int     useRandb[11];
int     useFile = 0;
int     useSize = 0;
int     sizeDMA = 0;
int     phase = 0;
int     maxRand;
int     sizeRand;
BOOL    writePhase = TRUE;
int     DMAbytes;
BOOL    quickTest = FALSE;
BOOL    filesWritten = FALSE;

HDC     hdcText = NULL;
HBITMAP hbmpText = NULL;
BITMAP  bmpText;
TEXTMETRIC tmt;
int     charHeight;
int     charWidth;
PLOGFONT  plf;
HFONT   hfont;

int   nPtSize;   
int  thissize;
char thisdata[200];



BOOL WINAPI CountThread()
{
    readme(); 
    threadCount = threadCount + 1;
    if (!goThread) 
    {
        return TRUE;
    }
    return FALSE; // exit here until Windows closes
}

BOOL CalibrateTimer()
{
    int i, j;

    BOOL done = FALSE;
    goThread = TRUE;    
    for (j=0; j<threadPass; j++)
    {     
        threadCount = 0;
        getStartTime();
        do
        {
            for (i=0; i<5; i++)
            {
                if(CountThread()) done = TRUE;
            }       
            getRunSecs();
        }
        while (runSecs < 0.2);
        countsPerSec = (double)threadCount / runSecs;
        if(countsPerSec > maxCount) maxCount = countsPerSec;
        countsPerSec = maxCount;        
    }
    return done;
}


BOOL StartThread()
{
    DWORD dwThreadId;

    hThreadHandle = CreateThread(
                        NULL,
                        0,
                        CountLoop,
                        inputParam,
                        0,
                        &dwThreadId
                        );
    if ( hThreadHandle == NULL ) 
    {
        return FALSE;
    }   
   return TRUE;
}        

DWORD WINAPI CountLoop(LPVOID inParam)
{
    BOOL threadNeeded = TRUE;
    inParam = inParam;
    
    while (threadNeeded)
    {
        if (CountThread()) threadNeeded = FALSE;
        // FALSE only when Windows closes
    }
    return 0;
}


/*
 * WinMain - initialization, message loop
 */
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    int         size1, min1, max1, files1;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    goThread = TRUE;
    StartThread();    
    SetThreadPriority(hThreadHandle, THREAD_PRIORITY_IDLE);
    SuspendThread(hThreadHandle);

    testFirst();
    useDMA = FALSE;
    useRand = FALSE;

    testStarted = TRUE;
    reView(blksizes, blksizes);
    if (autoRun)
    {
        phase = 1;
        CleanUpAndPostQuit();
    }

    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (phase > 0)
        {
            if (phase == 1)
            {
                if (quickTest)
                {
                    size1 = grafmbytes;
                    grafmbytes = 8;
                    if (!testStarted) grafmbytes = 4;
                    files1 = totalFiles;
                    totalFiles = 2;
                    min1 = minBlockChoice;
                    max1 = maxBlockChoice;
                    minBlockChoice = 6;
                    maxBlockChoice = 6;
                }
                if (startTests())
                {
                    phase = 2;
                    useSize = minBlockChoice;
                    useFile = 0;
                    writePhase = TRUE;
                }
                else
                {
                    phase = 0;
                    SetCursor(LoadCursor(NULL, IDC_ARROW));
                }
            }
            else if (phase == 2)
            {
                if (writePhase)
                {
                    if (write1File())
                    {
                        useFile = useFile + 1;
                        if (useFile == totalFiles)
                        {
                            useFile = 0;
                            writePhase = FALSE;
                        }
                    }
                    else
                    {
                        phase = 5;
                    }
                }
                else
                {
                    if (read1File())
                    {
                        useFile = useFile + 1;
                        if (useFile == totalFiles)
                        {
                            useFile = 0;
                            writePhase = TRUE;
                            useSize = useSize + 1;
                            if (useSize > maxBlockChoice)
                            {
                                phase = 3;
                                sizeDMA = 0;
                                DMAbytes = 1024;
                                viewDMA = TRUE;
                            }
                            else
                            {
                                eraseFiles();
                            }
                                
                        }
                    }
                    else
                    {
                        phase = 5;
                    }
                }                
            }
            else if (phase == 3)
            {
                if (read1DMA())
                {
                    sizeDMA = sizeDMA + 1;
                    DMAbytes = DMAbytes * 2;
                    if (sizeDMA == blksizes)
                    {
                        phase = 4;
                        viewDMA = FALSE;
                        initRand();
                        viewRand = TRUE;
                    }
                }
                else
                {
                    viewDMA = FALSE;
                    phase = 5;
                }
            }
            else if (phase == 4)
            {
                if (read1Rand())
                {
                    sizeRand = sizeRand - 1;
                    if (sizeRand < 0)
                    {
                        eraseFiles();
                        phase = 5;
                        viewRand = FALSE;
                    }
                }
                else
                {
                    phase = 5;
                }
            }
            else if (phase == 5)
            {
                endTests();
                phase = 0;
                if (quickTest)
                {
                    minBlockChoice = min1;
                    maxBlockChoice = max1;
                    totalFiles = files1;
                    grafmbytes = size1;
                    quickTest = FALSE;
                }
            }
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }

    ResumeThread(hThreadHandle);
    goThread = FALSE;
    CloseHandle( hThreadHandle );

    return msg.wParam;
    
} /* WinMain */

/****************************************************************************/
/*                   Initialization and object creation                     */
/****************************************************************************/
/*
 * InitApp
 * Creates window
 */
static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;
    HDC  hdc;
    LPCTSTR  dir = NULL;
    int i, j, gb, w;
    char *delims = {" ,"};
    char commands[20][256];
    char *p;
    char thisPart[20] = "  ";
    char buf[1024] = "";
    int numberFiles;
    int mbytes;
    
    /*
     * set up and registers the window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    /*
     * Create the window
     */

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPEDWINDOW, // | WS_HSCROLL | WS_VSCROLL,
            0,
            0,
            screenWide,
            screenHigh,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    strcpy(commandLine, strupr(cmdline));

    i = 0;
    p = strtok(commandLine, delims);
    while (p != NULL)
    {
        strcpy(commands[i], p);
        strcat(buf, commands[i]);
        strcat(buf, " ");
        i = i + 1;
        if (i > 19) break;
        p = strtok(NULL, delims);
    }
    strcpy(commandLine, buf);
    if (i > 0)
    {
        for (j=0; j<i; j++)
        {
            switch(commands[j][0])
            {            
                case 'A': // AutoRun
                 autoRun = TRUE;
                 break;
                
                case 'F':  // FilesToWrite
                 if (j < i -1)
                 {
                     sscanf(commands[j+1], "%d", &numberFiles);
                     if (numberFiles < 1 || numberFiles > maxfiles)
                     {
                         commandError = TRUE;
                     }
                     else
                     {
                         totalFiles = numberFiles;
                         autoRun = TRUE;
                     }
                 }
                 else
                 {
                     commandError = TRUE;
                 }
                 break;

                case 'M':  // Mbytes
                 if (j < i -1)
                 {
                     sscanf(commands[j+1], "%d", &mbytes);
                     if (mbytes < 1 || mbytes > maxMbytes)
                     {
                         commandError = TRUE;
                     }
                     else
                     {
                        grafmbytes = mbytes;
                        autoRun = TRUE;
                     }
                 }
                 else
                 {
                     commandError = TRUE;
                 }
                 break;
            }
        }
        if (commandError) autoRun = FALSE;
    }
    WhatConfig();
    GetDiskStats();
    
    GetCurrentDirectory(MAX_PATH, currentDir);

    if (strlen(currentDir) < 2)
    {
        strcpy (thisPart, currentDir);
    }
    else
    {
        strncpy(thisPart, currentDir, 2);
    }
    sprintf(diskPart, "%sDisk/partition", thisPart);

    gb = 1;
    for (i=0; i<blksizes; i++)
    {
        gBlockSize[i] = gb;
        gb = gb * 2;
        for (j=0; j<maxfiles; j++)
        {
            GWriteMBPS1[i][j] = 0;
            GReadMBPS1[i][j]  = 0;
            GWriteUt1[i][j] = 0;
            GReadUt1[i][j]  = 0;
        }
    }
    for (i=0; i<11; i++)
    {
         DMAcpuUt[i] = 0;
         DMAMBPS[i] = 0;
    }
    useDMA = FALSE;
    for (i=0; i<11; i++)
    {
         RandcpuUt[i] = 0;
         RandMsecs[i] = 0;
    }
    useRand = FALSE;
    minBlockChoice = 0;
    maxBlockChoice = 10;

    srand(123);
    for (i=0; i<1000; i++)
    {
        randomNumber[i] = (double) rand() / 32767;
    }

    // For rotated Y-Axis labels
    
    hdc = GetDC(mainHWND);
    hbmpText = CreateCompatibleBitmap(hdc, 200, 60);
    GetObject(hbmpText, sizeof(BITMAP), &bmpText);
    hdcText = CreateCompatibleDC(hdc);
    SelectObject(hdcText, hbmpText);
    ReleaseDC(mainHWND, hdc);

    // Font
    
    plf = (PLOGFONT) LocalAlloc(LPTR, sizeof(LOGFONT));
    nPtSize = 8;
    plf->lfHeight = -MulDiv(nPtSize, GetDeviceCaps(hdcText, LOGPIXELSY), 72);
    strcpy(plf->lfFaceName, "Courier New");
    hfont = CreateFontIndirect(plf);
    SelectObject(hdcText, hfont);

    GetTextMetrics(hdcText, &tmt);
    charHeight = tmt.tmHeight;
    charWidth  = tmt.tmAveCharWidth;

    // Y-Axis labels
    
    SetBkColor(hdcText, RGB(255,255,255));
    SetTextColor(hdcText, RGB(0,0,0)); 

    sprintf(thisdata, "CPU Utilisation ");
    thissize = 16;
    ExtTextOut(hdcText, 0, 40, ETO_CLIPPED, NULL, thisdata, thissize, (LPINT) NULL);
    
    sprintf(thisdata, "Millisecs Read Time  ");
    w = 20;
    thissize = 21;
    ExtTextOut(hdcText, 0, w, ETO_CLIPPED, NULL, thisdata, thissize, (LPINT) NULL);
    sprintf(thisdata, "MegaBytes Per Second  ");
    w = 0;
    ExtTextOut(hdcText, 0, w, ETO_CLIPPED, NULL, thisdata, thissize, (LPINT) NULL);


    LocalFree((HLOCAL) plf);
    DeleteObject(hfont);

    // Display the window
    
    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);

    return TRUE;
    
}  // InitApp


/*
 * WindowProc - handle messages for the main application window
 */

long FAR PASCAL WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
    char   tempFile[256];
    PAINTSTRUCT      ps;
    HDC  hdc;
    
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:    
        CreateWindow("BUTTON", "Exit", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                     butL, butT + 3 * butH, butW, butH, hwnd, (HMENU)FINISH_GRAPH, MyInstance, NULL);  
        CreateWindow("BUTTON", "Run", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                     butL, butT + 2 * butH, butW, butH, hwnd, (HMENU)RUN_BUTTON, MyInstance, NULL);  
        CreateWindow("BUTTON", "Quick Test", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                     butL, butT + 1 * butH, butW, butH, hwnd, (HMENU)QUICK_RUN, MyInstance, NULL);  
        CreateWindow("BUTTON", "Variables", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                     butL, butT, butW, butH, hwnd, (HMENU)PARAM_GRAPH, MyInstance, NULL);
        return 0;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_PAINT:
         InvalidateRect(hwnd, NULL, TRUE);
         hdc = BeginPaint(hwnd, &ps);
         if (testStarted) drawGraphs(blksizes, blksizes, hdc); 
         EndPaint(hwnd, &ps);
         break;
       
    case WM_KEYDOWN:
        break;

    case WM_ACTIVATE:
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {
        case MENU_ABOUT:
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            break;

        case PARAM_GRAPH:
            DialogBox( MyInstance,"DIALOG_21", hwnd, (DLGPROC)RunOptions );
            reView(blksizes, blksizes);                   
            break;

        case MENU_PARAM:
            DialogBox( MyInstance,"DIALOG_21", hwnd, (DLGPROC)RunOptions );
            reView(blksizes, blksizes);                   
            break;

        case MENU_RUNGRAPH:
            phase = 1;
            break;
            
        case MENU_QUICKTEST:
            quickTest = TRUE;
            phase = 1;    
            break;

        case MENU_LOAD:
            loadIt = TRUE;
            if (GetFileName())
            {
                FindFirstFile(grafFile, &W32DiskData);
                strcpy (fileToRead, W32DiskData.cFileName);
                loadFile();
                SetCurrentDirectory(currentDir);
            }
            break;
   
        case MENU_SAVE:
            loadIt = FALSE;
            strcpy(tempFile, bmpfile);
            if (GetFileName())
            {
                Sleep(500);
                reView(blksizes, blksizes);
                saveBmp();
                SetCurrentDirectory(currentDir);
                strcpy(bmpfile, tempFile);
            }
            break;

        case RUN_BUTTON:
            phase = 1;    
            break;

        case QUICK_RUN:
            quickTest = TRUE;
            phase = 1;    
            break;
            
        case MENU_VIEWGRAPH:
            viewDMA = FALSE;
            viewRand = FALSE;
            reView(blksizes, blksizes);
            break;

        case MENU_DMAGRAPH:
            if (useDMA)
            {
                viewDMA = TRUE;
                viewRand = FALSE;
            }
            else
            {
                MessageBox(NULL, "No DMA Results", "Warning", MB_ICONWARNING | MB_OK);
            }
            reView(blksizes, blksizes);
            break;

        case MENU_RANDGRAPH:
            if (useRand)
            {
                viewRand = TRUE;
                viewDMA = FALSE;
            }
            else
            {
                MessageBox(NULL, "No Random Results", "Warning", MB_ICONWARNING | MB_OK);
            }
            reView(blksizes, blksizes);
            break;

        case MENU_HELP:
            DialogBox( MyInstance,"DIALOG_44", hwnd, (DLGPROC)AboutDlgProc );                   
            break;
            
        case FINISH_GRAPH:
            CleanUpAndPostQuit();
            break;                

        case MENU_EXIT:
            CleanUpAndPostQuit();
            break;

            default:
            return DefWindowProc( hwnd, msg, wparam, lparam );                
        }
        break;
        
    default:   
    return DefWindowProc( hwnd, msg, wparam, lparam );
    } 
    return 0;

} /* WindowProc */



/*
 * AboutDlgProc - processes messages for the about dialog.
 */
BOOL  FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} /* AboutDlgProc */


/*
 * CleanUpAndPostQuit
 * Release all D3DRM objects, post a quit message and set the bQuit flag
 */
void
CleanUpAndPostQuit(void)
{
    bQuit = TRUE;
    if (hdcText) DeleteDC(hdcText);
    if (hbmpText) DeleteObject(hbmpText);
    if (myData) VirtualFree(myData, 0, MEM_RELEASE);
    if (filesWritten) eraseFiles();
    return;
}


BOOL  FAR PASCAL UpdateRadioButtons( HWND hdlg)
{
    if(useCache)
    {
        SendDlgItemMessage(hdlg, USE_CACHE, BM_SETCHECK, TRUE, 0);
    }
    else
    {
        SendDlgItemMessage(hdlg, USE_CACHE, BM_SETCHECK, FALSE, 0);
    }
    if(measureCPU)
    {
        SendDlgItemMessage(hdlg, MEASURE_UT, BM_SETCHECK, TRUE, 0);
    }
    else
    {
        SendDlgItemMessage(hdlg, MEASURE_UT, BM_SETCHECK, FALSE, 0);
    }
    if(largeRAM)
    {
        SendDlgItemMessage(hdlg, BIG_RAM, BM_SETCHECK, TRUE, 0);
    }
    else
    {
        SendDlgItemMessage(hdlg, BIG_RAM, BM_SETCHECK, FALSE, 0);
    }
    return TRUE;
}

BOOL  FAR PASCAL UpdateCombos( HWND hdlg, int minb, int maxb)
{
    minBlockChoice = minb;
    maxBlockChoice = maxb;
    if(minb > maxb)
    {
        maxBlockChoice = minb;
    }
    SendDlgItemMessage(hdlg, MINBLOCKSIZE, CB_SETCURSEL, minBlockChoice, 0L);
    SendDlgItemMessage(hdlg, MAXBLOCKSIZE, CB_SETCURSEL, maxBlockChoice, 0L);

    return TRUE;
}

BOOL  FAR PASCAL RunOptions( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    HWND     hwndName;
    int     tempint;
    int     tempfiles;
    int     wTextWidth;
    char    temptext[256];
    BOOL     dataOK = TRUE;
    int     i, minb, maxb;
    char    wdata[10];
    char    msg2[200];

    minb = minBlockChoice;
    maxb = maxBlockChoice ;
    
    switch( msg )
     {
       case WM_INITDIALOG:
        SetDlgItemInt(hdlg, GRAPH_MB,  grafmbytes, FALSE); 
        SetDlgItemInt(hdlg, NUMBERFILES,  totalFiles, FALSE); 

        for (i=0; i<blksizes; i++) 
        {
           sprintf(wdata,"%4d", gBlockSize[i]);
           SendDlgItemMessage(hdlg, MINBLOCKSIZE, CB_ADDSTRING, 0, (LPARAM) wdata);
           SendDlgItemMessage(hdlg, MAXBLOCKSIZE, CB_ADDSTRING, 0, (LPARAM) wdata);
        }
        UpdateCombos(hdlg, minb, maxb);

        hwndName = GetDlgItem(hdlg, GRAPH_FILE);
        SendMessage(hwndName, WM_SETTEXT, 0,
                                    (LPARAM) ((LPSTR) bmpfile));
        hwndName = GetDlgItem(hdlg, NOTE_1);
        SendMessage(hwndName, WM_SETTEXT, 0,
                                    (LPARAM) ((LPSTR) note1));
        hwndName = GetDlgItem(hdlg, NOTE_2);
        SendMessage(hwndName, WM_SETTEXT, 0,
                                    (LPARAM) ((LPSTR) note2));
        hwndName = GetDlgItem(hdlg, NOTE_3);
        SendMessage(hwndName, WM_SETTEXT, 0,
                                    (LPARAM) ((LPSTR) note3));
        UpdateRadioButtons(hdlg);

        return( TRUE );

      case WM_COMMAND:
        if( LOWORD( wparam ) ==  DO_RUN)
        {
             hwndName = GetDlgItem(hdlg, NOTE_1);
             wTextWidth = SendMessage(hwndName, WM_GETTEXT,
                      sizeof(temptext), (LPARAM) ((LPSTR) temptext));
             if (wTextWidth > 0)
             {
                  lstrcpy(note1, temptext);                
             }
             hwndName = GetDlgItem(hdlg, NOTE_2);
             wTextWidth = SendMessage(hwndName, WM_GETTEXT,
                      sizeof(temptext), (LPARAM) ((LPSTR) temptext));
             if (wTextWidth > 0)
             {
                  lstrcpy(note2, temptext);                
             }
             hwndName = GetDlgItem(hdlg, NOTE_3);
             wTextWidth = SendMessage(hwndName, WM_GETTEXT,
                      sizeof(temptext), (LPARAM) ((LPSTR) temptext));
             if (wTextWidth > 0)
             {
                  lstrcpy(note3, temptext);                
             }

             hwndName = GetDlgItem(hdlg, GRAPH_FILE);
             wTextWidth = SendMessage(hwndName, WM_GETTEXT,
                      sizeof(temptext), (LPARAM) ((LPSTR) temptext));
             if (wTextWidth > 0)
             {
                  lstrcpy(bmpfile, temptext);
             }
             else
             {
                 dataOK = FALSE;
             }
             tempfiles = GetDlgItemInt(hdlg, NUMBERFILES, NULL, FALSE);
             if (tempfiles < 1)
             {
                 tempfiles = totalFiles;
             }
             tempint = GetDlgItemInt(hdlg, GRAPH_MB, NULL, FALSE);
             if (tempint > 0 && tempint < maxMbytes+1)
             {
                 if ((double)tempint * (double)tempfiles + 10.0 > freeDiskMB)
                 {
                    sprintf (fileMessage, "%d files at %d MB + 10 MB too big\n"
                                          "Only %8.1fMB free on disk\n"
                                          "Value not changed",
                                           tempfiles, tempint,  freeDiskMB);
                    MessageBox(NULL, fileMessage, "Error", MB_ICONWARNING | MB_OK);
                    EndDialog( hdlg, TRUE );
                    return (FALSE);
                 }
                 else
                 {
                    grafmbytes = tempint;
                 }
             }
             else
             {
                 dataOK = FALSE;
             }
             tempint = GetDlgItemInt(hdlg, NUMBERFILES, NULL, FALSE);
             if (tempint > 0 && tempint < maxfiles+1)
             {
                 totalFiles = tempint;
             }
             else
             {
                 dataOK = FALSE;
             }
             
             if (grafmbytes  > ramsize / 1024 / 1024)
             {
                 if (largeRAM)
                 {
                     sprintf (fileMessage, "Memory data size same as file size\n"
                                           "File %d MB, RAM size %d MB\n"
                                           "WARNING: Excessive paging might occur",
                                            grafmbytes, ramsize / 1024 / 1024);
                     if (grafmbytes  > ramsize / 1024 / 1024 * 2 || grafmbytes > 2048)
                     {
                        sprintf (msg2, "\nPROGRAM RESTRICTION MB > 2 x RAM or > 2048 MB\n"
                                       "File size as data size turned off");
                        largeRAM = FALSE;
                        strcat(fileMessage, msg2);
                     }                       
                     MessageBox(NULL, fileMessage, "WARNING", MB_ICONWARNING | MB_OK);
                 }
             }
             
             if (dataOK)
             {
                EndDialog( hdlg, TRUE );
                return (TRUE);
             }
             else
             {
                MessageBox(NULL, " Parameters wrong. Try again", "Error", MB_ICONWARNING | MB_OK);
                EndDialog( hdlg, TRUE );
                return (FALSE);
             }                 
        }
        else if( LOWORD( wparam ) == IDCANCEL )
        {
             EndDialog( hdlg, TRUE );    
             return (FALSE);
        }
        else if( HIWORD( wparam ) == CBN_SELCHANGE)
        {
            switch (LOWORD (wparam))
            {
                case MINBLOCKSIZE:
                  minb = (int)SendDlgItemMessage(hdlg, MINBLOCKSIZE, CB_GETCURSEL, 0, 0);
                  break;
        
                case MAXBLOCKSIZE:
                  maxb = (int)SendDlgItemMessage(hdlg, MAXBLOCKSIZE, CB_GETCURSEL, 0, 0);
                  break;
            }
            UpdateCombos(hdlg, minb, maxb);
        }
        else if ( HIWORD( wparam ) == BN_CLICKED)
        {
             switch (LOWORD (wparam))
              {
                  case USE_CACHE:
                    if(useCache)
                    {
                        useCache = FALSE;
                    }
                    else
                    {
                        useCache = TRUE;
                    }
                    break;
         
                  case MEASURE_UT:
                    if(measureCPU)
                    {
                        measureCPU = FALSE;
                    }
                    else
                    {
                        measureCPU = TRUE;
                    } 
                    break;

                  case BIG_RAM:
                    if(largeRAM)
                    {
                        largeRAM = FALSE;
                    }
                    else
                    {
                        largeRAM = TRUE;
                    }
                    break;
              }
             UpdateRadioButtons(hdlg);
        }
     }
     return( FALSE );

} // RunOptions


BOOL  drawGraphs(int nw, int nr, HDC hdc)
{
    HBRUSH hbr;
    HPEN hpen;

    int   text2 =  5;   
    int   yTop1 =    20;  // 20
    int   yBottom1 = 230; // 230
    int   yInc1;
    int   scale1;
    int   height1;
    int   xStart = 90;  // 90;
    int   xEnd   = 480; // 480; 
    int   yTop2 =    246; // 256
    int   yBottom2 = 376; // 386
    int   yInc2;
    int   scale2;
    int   height2;
    int   yTitle1 = 400;
    int   i, j, m, v, n, g, w;
    int   s1, s2;
    int   l1, l2;
    int   pixelsUp;
    double dmaspd;
    double randMin;
    double randMax;
    char   randMins[10];
    char   randMaxs[10];
    int    mMax = 10;
    int    uMax = 10;

        
    height1 = yBottom1 - yTop1;
    height2 = yBottom2 - yTop2;
    yInc1 = height1/10;
    yInc2 = height2/10;

    if (fromLog)
    {
        strcpy(versionIs, versionIn);
    }
    else
    {
        strcpy(versionIs, version);
    }        
    sprintf(title, "Roy Longbottom's Disk Write and Read Benchmark %s", versionIs);
    sprintf(title2, "Roy Longbottom's Disk DMA Read Benchmark %s      ", versionIs);
    sprintf(title3, "Roy's 1KB Block Random Read Benchmark %s            ", versionIs);

    GetClientRect(mainHWND, &recwin);
    j = (recwin.right + 3) / 4;
    xtraWide = j * 4  - recwin.right;
    if (xtraWide == 1) recwin.right = recwin.right - 3;
    if (xtraWide == 2) recwin.right = recwin.right - 2;
    if (xtraWide == 3) recwin.right = recwin.right - 1;

    // Data to display as graphs GReadMBPS2[][] and GReadUt2[][]
    if (viewDMA)
    {
        for (i=0; i<blksizes; i++)
        {
            for (j=0; j<maxfiles; j++)
            {
                GWriteMBPS2[i][j] = 0;
                GReadMBPS2[i][j] = 0;
                GWriteUt2[i][j] = 0;
                GReadUt2[i][j] = 0;
            }
        }
        for (i=0; i<11; i++)
        {
            GReadMBPS2[i][0] = DMAMBPS[i];
            GReadUt2[i][0] = DMAcpuUt[i];
        }
    }
    else if (viewRand)
    {
        for (i=0; i<blksizes; i++)
        {
            for (j=0; j<maxfiles; j++)
            {
                GWriteMBPS2[i][j] = 0;
                GReadMBPS2[i][j] = 0;
                GWriteUt2[i][j] = 0;
                GReadUt2[i][j] = 0;
            }
        }
        for (i=0; i<11; i++)
        {
            GReadMBPS2[i][0] = RandMsecs[i];
            GReadUt2[i][0] = RandcpuUt[i];
        }
    }
    else
    {
        for (i=0; i<blksizes; i++)
        {
            for (j=0; j<maxfiles; j++)
            {
                GWriteMBPS2[i][j] = GWriteMBPS1[i][j];
                GReadMBPS2[i][j] = GReadMBPS1[i][j];
                GWriteUt2[i][j] = GWriteUt1[i][j];
                GReadUt2[i][j] = GReadUt1[i][j];
            }
        }
    }                  

    // Find maximum values
    
    for (i=0; i<blksizes; i++)
    {
        for (j=0; j<totalFiles; j++)
        {
            if ((int)(GWriteMBPS2[i][j]+0.5) > mMax) mMax = (int)(GWriteMBPS2[i][j]+0.5);
            if ((int)(GReadMBPS2[i][j]+0.5)  > mMax) mMax = (int)(GReadMBPS2[i][j]+0.5);
            if ((int)(GWriteUt2[i][j]+0.5)   > uMax) uMax = (int)(GWriteUt2[i][j]+0.5);
            if ((int)(GReadUt2[i][j]+0.5)    > uMax) uMax = (int)(GReadUt2[i][j]+0.5);
        }
    }

    // MB/sec adjust scale
    
    n = 6;
    m = 1;
    for (i=0; i<n; i++)
    {
        if (mMax < 100 * m + 1)
        {
            v = mMax;
            if ((v / 10 / m) * 10 * m != v)
            {
                v = v + 10 * m;
                v = (v / 10 / m * 10 * m);
            }
            scale1 = v;
            n = i;
        }
        m = m * 10;
    }

    // CPUt adjust scale
    
    n = 2;
    m = 1;
    if (uMax > 50) uMax = 100;
    for (i=0; i<n; i++)
    {
        if (uMax < 100 * m + 1)
        {
            v = uMax;
            if ((v / 10 / m) * 10 * m != v)
            {
                v = v + 10 * m;
                v = (v / 10 / m * 10 * m);
            }
            scale2 = v;
            n = i;
        }
        m = m * 10;
    }


    // Fill most of screen white

    hbr = CreateSolidBrush(RGB(255,255,255));  
    SelectObject(hdc, hbr);
    hpen = CreatePen(PS_SOLID, 0, RGB(255,255,255));
    SelectObject(hdc, hpen);
    Rectangle(hdc, recwin.left, recwin.top, recwin.right, butT-1);
    Rectangle(hdc, recwin.left, butT + 4 * butH, recwin.right, recwin.bottom);
    Rectangle(hdc, recwin.left, butT-1, butL-1, butT + 4 * butH);
    DeleteObject(hpen);
    
    // Text black on white

    SetBkColor(hdc, RGB(255,255,255));
    SetTextColor(hdc, RGB(0,0,0)); 

    // Heading Font
    
    plf = (PLOGFONT) LocalAlloc(LPTR, sizeof(LOGFONT));
    nPtSize = 10; // 12
    plf->lfHeight = -MulDiv(nPtSize, GetDeviceCaps(hdc, LOGPIXELSY), 72);
    strcpy(plf->lfFaceName, "Times New Roman");
    hfont = CreateFontIndirect(plf);
    SelectObject(hdc, hfont);

    GetTextMetrics(hdc, &tmt);
    charHeight = tmt.tmHeight;
    charWidth  = tmt.tmAveCharWidth;

    // Heading
 
    if (readWrite)
    {
        sprintf(thisdata, "Disk Test Running - Press Esc Key To Kill If Running Time Too Long                   ");
    }
    else
    {
        if (viewDMA)
        {
            sprintf(thisdata, "%s   ", title2);
        }
        else if (viewRand)
        {
            sprintf(thisdata, "%s   ", title3);
        }
        else
        {
            sprintf(thisdata, "%s   ", title);
        }
    }
    thissize =  strlen(thisdata);
    ExtTextOut(hdc, xStart, 1, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    // Maximum DMA speeds and max/min random access speeds
    
    if (useDMA)
    {
        dmaspd = 0;
        for (i=0; i<11; i++)
        {
            if (DMAMBPS[i] > dmaspd) dmaspd = DMAMBPS[i];
        }
        SetTextColor(hdc, RGB(0,0,255)); 
        sprintf(thisdata, "DMA %6.1f MB/sec", dmaspd);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 20, yTop1 - 18, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        randMin = 9999999;
        randMax = 0;
        for (i=0; i<11; i++)
        {
            if (RandMsecs[i] > randMax) randMax = RandMsecs[i];
            if (RandMsecs[i] > 0)
            {
                if (RandMsecs[i] < randMin) randMin = RandMsecs[i];
            }
        }
        if (randMin == 9999999) randMin = 0;
        if (randMin < 0.01)
        {
            sprintf(randMins, "%5.3f", randMin);
        }
        else if (randMin < 1.0)
        {
            sprintf(randMins, "%4.2f", randMin);
        }
        else if (randMin < 10.0)
        {
            sprintf(randMins, "%3.1f", randMin);
        }
        else if (randMin < 100.0)
        {
            sprintf(randMins, "%4.1f", randMin);
        }
        else
        {
            sprintf(randMins, "%d", (int)randMin);
        }
        if (randMax < 0.01)
        {
            sprintf(randMaxs, "%5.3f", randMax);
        }
        else if (randMax < 1.0)
        {
            sprintf(randMaxs, "%4.2f", randMax);
        }
        else if (randMax < 10.0)
        {
            sprintf(randMaxs, "%3.1f", randMax);
        }
        else if (randMax < 100.0)
        {
            sprintf(randMaxs, "%4.1f", randMax);
        }
        else
        {
            sprintf(randMaxs, "%d", (int)randMax);
        }        

        sprintf(thisdata, "Rand %s - %s ms", randMins, randMaxs);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 20, yTop1 - 3, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);        
    }
    else
    {
        sprintf(thisdata, "                          ");
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 20, yTop1 - 18, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        ExtTextOut(hdc, xEnd + 20, yTop1 - 3, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }
    SetTextColor(hdc, RGB(0,0,0)); 
    

    LocalFree((HLOCAL) plf);
    DeleteObject(hfont);

    // Font
    
    plf = (PLOGFONT) LocalAlloc(LPTR, sizeof(LOGFONT));
    nPtSize = 8;
    plf->lfHeight = -MulDiv(nPtSize, GetDeviceCaps(hdc, LOGPIXELSY), 72);
    strcpy(plf->lfFaceName, "Courier New");
    hfont = CreateFontIndirect(plf);
    SelectObject(hdc, hfont);

    GetTextMetrics(hdc, &tmt);
    charHeight = tmt.tmHeight;
    charWidth  = tmt.tmAveCharWidth;

    hpen = CreatePen(PS_SOLID, 0, RGB(0,0,0));
    SelectObject(hdc, hpen);
     

    // key titles
    if (viewRand)
    {
        sprintf(thisdata, "MB files   ");
    }
    else
    {
        sprintf(thisdata, "KB blocks  ");
    }
    thissize = 10;
    ExtTextOut(hdc, xEnd + 10, yBottom1+1, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    sprintf(thisdata, "Write  ");
    thissize = 6;
    ExtTextOut(hdc, xEnd + 10, yBottom1 + 25, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    sprintf(thisdata, "Read  ");
    thissize = 5;
    ExtTextOut(hdc, xEnd + 75, yBottom1 + 25, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    // Colour keys

    SetBkColor(hdc, RGB(255,0,0));
    sprintf(thisdata, "  ");
    thissize = 2;
    ExtTextOut(hdc, xEnd + 50, yBottom1 + 25, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    SetBkColor(hdc, RGB(0,0,255));
    sprintf(thisdata, "  ");
    thissize = 2;
    ExtTextOut(hdc, xEnd + 110, yBottom1 + 25, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    // MB/sec Box
    
    MoveToEx(hdc, xStart, yTop1, NULL);
    LineTo(hdc, xStart, yBottom1);
    LineTo(hdc, xEnd, yBottom1);
    LineTo(hdc, xEnd, yTop1);
    LineTo(hdc, xStart-3, yTop1);

    SetBkColor(hdc, RGB(255,255,255));

    //  MB/sec scale
    
    s1 = 2;
    if (scale1 > 99) s1 = 3;
    if (scale1 > 999) s1 = 4;
    if (scale1 > 9999) s1 = 5;
    if (scale1 > 99999) s1 = 6;
    if (scale1 > 999999) s1 = 7;
    sprintf(thisdata, "%d", scale1); 
    l1 = strlen(thisdata);
    for (i=1; i<11; i++)
    {
        sprintf(thisdata, "%d", i*scale1/10);
        if (lstrlen(thisdata) < l1) sprintf(thisdata, " %d", i*scale1/10);
        strcat(thisdata, " ");
        thissize = s1;
        ExtTextOut(hdc, xStart - (s1 + 1) * charWidth, yBottom1 - i * yInc1 - charHeight / 2,
                   ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }

    // Y-Axis labels
    // Rotate

    w = 0;
    if (viewRand) w = 20;

    for(i=0; i<charWidth*21; i++)
    {
        for(j=0; j<charHeight; j++)
        {
            SetPixel(hdc, xStart-s1*charWidth-25+j, yBottom1-45-i, GetPixel(hdcText, i, j+w));
        }
    }

    for(i=0; i<charWidth*16; i++)
    {
        for(j=0; j<charHeight; j++)
        {
            SetPixel(hdc, xStart-s1*charWidth-25+j, yBottom2-20-i, GetPixel(hdcText, i, j+40));
        }
    }


    // KB block sizes

    sprintf(thisdata, "  1    2    4    8    16   32   64  128  256  512  1024   ");
    thissize = 56;
    ExtTextOut(hdc, xStart, yBottom1+1, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);


    // CPUt Box
    
    MoveToEx(hdc, xStart, yTop2, NULL);
    LineTo(hdc, xStart, yBottom2);
    LineTo(hdc, xEnd, yBottom2);
    LineTo(hdc, xEnd, yTop2);
    LineTo(hdc, xStart-3, yTop2);

    // CPUt Scale
    
    s2 = 2;
    if (scale2 > 99) s2 = 3;
    sprintf(thisdata, "%d", scale2); 
    l2 = strlen(thisdata);
    for (i=1; i<11; i++)
    {
        sprintf(thisdata, "%d", i*scale2/10);
        if (lstrlen(thisdata) < l2) sprintf(thisdata, " %d", i*scale2/10);
        strcat(thisdata, " ");
        thissize = s2;
        ExtTextOut(hdc, xStart - (s2 + 1) * charWidth, yBottom2 - i * yInc2 - charHeight / 2,
                   ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }

    // Grid lines
    
    for (i=1; i<10; i++)
    {
       MoveToEx(hdc, xStart-3, yBottom1 - i * yInc1, NULL);
       LineTo(hdc, xEnd, yBottom1 - i * yInc1);
    }
    for (i=1; i<10; i++)
    {
       MoveToEx(hdc, xStart-3, yBottom2 - i * yInc2, NULL);
       LineTo(hdc, xEnd, yBottom2 - i * yInc2);
    }

    DeleteObject(hbr);
    DeleteObject(hpen);

    // Write graphs Red
    
    hpen = CreatePen(PS_SOLID, 0, RGB(255,0,0));
    SelectObject(hdc, hpen);
    g = 4;
    for (i=0; i<nw; i++)
    {
        for (j=0; j<maxfiles; j++)
        {
            pixelsUp = (int)((double)height1 / (double)scale1 * GWriteMBPS2[i][j]);
            MoveToEx(hdc, xStart+i*10+g+j, yBottom1, NULL);
            LineTo(hdc, xStart+i*10+g+j, yBottom1 - pixelsUp);
            pixelsUp = (int)((double)height2 / (double)scale2 * GWriteUt2[i][j]);
            MoveToEx(hdc, xStart+i*10+g+j, yBottom2, NULL);
            LineTo(hdc, xStart+i*10+g+j, yBottom2 - pixelsUp);
        }
        g = g + 25;
    }
    DeleteObject(hpen);

    // Read graphs Blue
    
    hpen = CreatePen(PS_SOLID, 0, RGB(0,0,255));
    SelectObject(hdc, hpen);
    g = 21;
    for (i=0; i<nr; i++)
    {
        for (j=0; j<maxfiles; j++)
        {
            pixelsUp = (int)((double)height1 / (double)scale1 * GReadMBPS2[i][j]);
            MoveToEx(hdc, xStart+i*10+g+j, yBottom1, NULL);
            LineTo(hdc, xStart+i*10+g+j, yBottom1 - pixelsUp);
            pixelsUp = (int)((double)height2 / (double)scale2 * GReadUt2[i][j]);
            MoveToEx(hdc, xStart+i*10+g+j, yBottom2, NULL);
            LineTo(hdc, xStart+i*10+g+j, yBottom2 - pixelsUp);
        }
        g = g + 25;
    }
    DeleteObject(hpen);

    hpen = CreatePen(PS_SOLID, 0, RGB(255,255,255));
    SelectObject(hdc, hpen);


    sprintf(thisdata, "Disk Type/Notes:");
    thissize = strlen(thisdata);
    ExtTextOut(hdc, xEnd + 20, yTop1 + 1 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    if (strlen(note1) > 0 || strlen(note2) > 0 || strlen(note3) > 0)
    {
        thissize = strlen(note1);
        ExtTextOut(hdc, xEnd + 20, yTop1 + 2 * charHeight, ETO_CLIPPED, &recwin, note1, thissize, (LPINT) NULL);
        thissize = strlen(note2);
        ExtTextOut(hdc, xEnd + 20, yTop1 + 3 * charHeight, ETO_CLIPPED, &recwin, note2, thissize, (LPINT) NULL);
        thissize = strlen(note3);
        ExtTextOut(hdc, xEnd + 20, yTop1 + 4 * charHeight, ETO_CLIPPED, &recwin, note3, thissize, (LPINT) NULL);
    } 

    // Variables

    if (viewDMA)
    {
        sprintf(thisdata, "   Variables:   ");
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  6 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Reread Block        ");
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  7 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "DMA Transfer?       ", grafmbytes);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  8 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Min Block %4d KB  ", 1);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  9 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Max Block %4d KB  ", 1024);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 + 10 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }
    else if (viewRand)
    {
        sprintf(thisdata, "   Variables:   ");
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  6 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Read 1KB Blocks     ");
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  7 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Random Read         ");
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  8 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Min File %4d MB   ", 1);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  9 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Max File %4d MB   ", grafmbytes);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 + 10 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }
    else
    {
        sprintf(thisdata, "   Variables:   ");
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  6 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Files     %4d   ", totalFiles);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  7 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Size      %4d MB", grafmbytes);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  8 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Min Block %4d KB", gBlockSize[minBlockChoice]);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 +  9 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        sprintf(thisdata, "Max Block %4d KB", gBlockSize[maxBlockChoice]);
        thissize = strlen(thisdata);
        ExtTextOut(hdc, xEnd + 18, yTop1 + 10 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    }
        
    if(useCache)
    {
        sprintf(thisdata, "Use File Cache ");
    }
    else
    {
        sprintf(thisdata, "File Cache N/A      ");
    }
    thissize = strlen(thisdata);
    ExtTextOut(hdc, xEnd + 18, yTop1 + 11 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    if(largeRAM)
    {
        sprintf(thisdata, "RAMSize = FileSize ");
    }
    else
    {
         sprintf(thisdata, "Minimum RAM Data   "); 
    }
    thissize = strlen(thisdata);
    ExtTextOut(hdc, xEnd + 18, yTop1 + 12 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    if(measureCPU)
    {
        sprintf(thisdata, "Measure CPU Ut       ");
    }
    else
    {
         sprintf(thisdata, "CPU Ut N/A          "); 
    }
    thissize = strlen(thisdata);
    ExtTextOut(hdc, xEnd + 18, yTop1 + 13 * charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    // Date and Configuration - bottom

    sprintf(thisdata, "%s", configData1);
    if (!displayOnly) local_time();
    strncat(thisdata, timeday, strlen(timeday)-1);
    thissize = strlen(thisdata);
    ExtTextOut(hdc, 20, yBottom2 + 4, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    sprintf(thisdata, "%s", configData2);
    thissize = strlen(thisdata);
    ExtTextOut(hdc, 20, yBottom2 + 4 + charHeight, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    
    sprintf(thisdata, "%s: %s %d MB, Free %d MB ",
                       configData3, diskPart, (int)totalDiskMB, (int)freeDiskMB );
    thissize = strlen(thisdata);
    ExtTextOut(hdc, 20, yBottom2 + 4 + charHeight*2, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);

    DeleteObject(hpen);

    LocalFree((HLOCAL) plf);
    DeleteObject(hfont);
    return TRUE;

} // drawGraphs


BOOL reView(int bl1, int bl2)
{
    HDC  hdc;

    hdc = GetDC(mainHWND);
    drawGraphs(bl1, bl2, hdc);
    ReleaseDC(mainHWND, hdc);
    return TRUE;
}

BOOL  write1File()
{
    HCURSOR hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
    useDMA = FALSE;
    viewDMA = FALSE;
    useRand = FALSE;
    viewRand = FALSE;
    fromLog = FALSE;

    numberBlocks  = grafmbytes * 1024 / gBlockSize[useSize];
    bytesPerBlock = gBlockSize[useSize] * 1024;
    wordsPerBlock = bytesPerBlock / 4;
    if (!writeGFiles(useSize+1, useSize))
    {
        strcpy(errorMsg, "Write ");
        strcat(errorMsg, fileMessage);
        if (autoRun)
        {
            saveErrors();
        }
        else
        {
            MessageBox(mainHWND, fileMessage, "Write Files",
                                MB_ICONWARNING | MB_OK);
        }
        eraseFiles();
        grafSaved = TRUE;
        SetCursor(hcur);
        return FALSE;
    }
    SetCursor(hcur);
    return TRUE;
    
} // write1File

BOOL read1File()
{
    HCURSOR hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
    if (!readGFiles(useSize+1, useSize+1))
    {
        strcpy(errorMsg, "Read ");
        strcat(errorMsg, fileMessage);
        if (autoRun)
        {
            saveErrors();
        }
        else
        {
           MessageBox(mainHWND, fileMessage, "Read Files",
                               MB_ICONWARNING | MB_OK);

        }
        eraseFiles();
        grafSaved = TRUE;
        SetCursor(hcur);
        return FALSE;
    }
    SetCursor(hcur);
    return TRUE;

} // read1File


BOOL read1DMA()
{
    HCURSOR hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
    if (!readDMA())
    {
        strcpy(errorMsg, "DMA Read ");
        strcat(errorMsg, fileMessage);
        if (autoRun)
        {
            saveErrors();
        }
        else
        {
           MessageBox(mainHWND, fileMessage, "DMA Read",
                               MB_ICONWARNING | MB_OK);
        }
        eraseFiles();
        grafSaved = TRUE;
        SetCursor(hcur);
        return FALSE;
    }
    SetCursor(hcur);
    return TRUE;
    
} // read1DMA

BOOL read1Rand()
{
    HCURSOR hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
    if (!readRAND())
    {
        strcpy(errorMsg, "Random Read ");
        strcat(errorMsg, fileMessage);
        if (autoRun)
        {
            saveErrors();
        }
        else
        {
           MessageBox(mainHWND, fileMessage, "Random Read",
                               MB_ICONWARNING | MB_OK);

        }
        eraseFiles();
        grafSaved = TRUE;
        SetCursor(hcur);
        return FALSE;
    }
    SetCursor(hcur);
    return TRUE;
    
} // read1Rand            


BOOL  writeGFiles(int nw, int nr)
{
    HANDLE hFile;
    char  buf[200];
    char  fileName[200];
    int   j;
    BOOL  writeIt;
    DWORD BytesOut;
    BOOL  chekW;
    DWORD  chekSum;

    if (freeDiskMB - (double)grafmbytes  < 10.0)
    {
       sprintf (fileMessage, " File %d not written only %8.1fMB free",
                                    useFile+1, freeDiskMB);
       return FALSE;
    }
    strcpy(buf, "Test");
    if (useFile < 100) strcat(buf, "0");
    if (useFile < 10 ) strcat(buf, "0");
    sprintf(fileName, "%s%d.tst", buf, useFile);

    if (useCache)
    {
        hFile = CreateFile(fileName, GENERIC_WRITE,
              FILE_SHARE_READ | FILE_SHARE_WRITE,
              NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    }
    else
    {
        hFile = CreateFile(fileName, GENERIC_WRITE,
              FILE_SHARE_READ | FILE_SHARE_WRITE,
              NULL, CREATE_ALWAYS,
              FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING, NULL);
    }
    if (hFile == INVALID_HANDLE_VALUE)
    {
       sprintf (fileMessage, " Cannot open write file %s", fileName);
       return FALSE;
    }
    sprintf(windowText, " Writing %s %d Mbytes at %d Bytes Per Block", fileName, grafmbytes, bytesPerBlock);
    SetWindowText(mainHWND, windowText);
    strcpy(fileNames[useFile], fileName);
            
    threadPass = 1;
    CalibrateTimer();
    if (measureCPU)
    {
        threadCount = 0;
        ResumeThread(hThreadHandle);    
        SetThreadPriority(hThreadHandle, THREAD_PRIORITY_IDLE);
    }
    chekW = 1;
    chekSum = 0;
    filesWritten = TRUE;
    getStartTime();          
    for ( j = 0; j<numberBlocks; j++)
    {
      writeIt = WriteFile(hFile,  &myData[j * wordsPerBlock * bigOne], bytesPerBlock, &BytesOut, NULL);
      if(writeIt == 0) chekW = 0;
      chekSum = chekSum + BytesOut;
      if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
      {
         CloseHandle(hFile);
         sprintf (fileMessage, " File %s - Esc Pressed ", fileName);
         sprintf(windowText, " Writing %s %d Mbytes ESCAPE Pressed", fileName, grafmbytes);
         SetWindowText(mainHWND, windowText);
         if (measureCPU) SuspendThread(hThreadHandle);                                          
         return FALSE;
      }
    }
    FlushFileBuffers(hFile);
    CloseHandle(hFile);
    if (chekW == 0 || chekSum != bytesPerBlock * numberBlocks)
    {
       sprintf (fileMessage,
          " File not written properly %s %d bytes written, code %d", fileName, chekSum, chekW);
       return FALSE;
    }
    
    getRunSecs();

    if (runSecs == 0) runSecs = 0.0000001;
    
    if (measureCPU)
    {
        SuspendThread(hThreadHandle);
        cpuUt[useFile] = 100.0 * ((runSecs - (double)threadCount / countsPerSec) / runSecs);  
        if (cpuUt[useFile] > 100) cpuUt[useFile] = 100;
        if (cpuUt[useFile] < 0.01) cpuUt[useFile] = 0;
    }
    else
    {
        cpuUt[useFile] = 0;
    }
    
    GWriteUt1[useSize][useFile] = cpuUt[useFile];
    GWriteMBPS1[useSize][useFile] = (double)grafmbytes / runSecs;

    GetDiskStats();
    if (testStarted) reView(nw, nr);
    return TRUE;
    
} // writeGFiles


BOOL initRand()
{
    DWORD useMB;
    DWORD maxMB;
    int i;

    if (!useCache)
    {
        sprintf(windowText, " Random Reading file %s at 1 to %d MBytes", fileNames[totalFiles-1], grafmbytes);
        SetWindowText(mainHWND, windowText);

        threadPass = 1;
        CalibrateTimer();
        bytesPerBlock = 1024;
        maxMB =  grafmbytes;
        if (maxMB > 1024) maxMB = 1024;
        useMB = 1;
        maxRand = 11;
        for (i=0; i<11; i++)
        {
            useRandb[i] = useMB;
            if (useMB > maxMB) maxRand = maxRand - 1;
            useMB = useMB * 2;
        }
        sizeRand = maxRand - 1;
    }
    return TRUE;
    
} // initRand


BOOL readRAND()
{
    HANDLE hFile;
    int r;
    BOOL  readIt;
    DWORD BytesIn;
    int blockReads = 100;
    DWORD randomPos[1000];
    DWORD useMB;

    if (!useCache)
    {
        hFile = CreateFile(fileNames[totalFiles-1], GENERIC_READ,
                 FILE_SHARE_READ | FILE_SHARE_WRITE,
                 NULL, OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS
                                    | FILE_FLAG_NO_BUFFERING, NULL);
        if (hFile == INVALID_HANDLE_VALUE)
        {
           sprintf (fileMessage, " Cannot open read file %s", fileNames[totalFiles-1]);
           return FALSE;
        }
        useMB = useRandb[sizeRand];
        numberBlocks = useMB * 1024 * 1024 / bytesPerBlock;
        for (r=0; r<blockReads; r++)
        {
            randomPos[r] = (int) (randomNumber[r]
                * (double)numberBlocks) * bytesPerBlock;
            if (randomPos[r] + bytesPerBlock > useMB * 1024 * 1024)
                randomPos[r] = useMB * 1024 * 1024 - bytesPerBlock;
        }       
        if (measureCPU)
        {
            threadCount = 0;
            ResumeThread(hThreadHandle);    
            SetThreadPriority(hThreadHandle, THREAD_PRIORITY_IDLE);
        }
        getStartTime();
        for (r=0; r<blockReads; r++)
        {
            SetFilePointer(hFile,  randomPos[r], NULL, FILE_BEGIN);
            readIt = ReadFile(hFile, &myData[0], bytesPerBlock, &BytesIn, NULL);
        }
        FlushFileBuffers(hFile);
        getRunSecs();
        if (runSecs == 0) runSecs = 0.00000001;   
        if (measureCPU)
        {
            SuspendThread(hThreadHandle);
            RandcpuUt[sizeRand] = 100.0 * ((runSecs - (double)threadCount / countsPerSec) / runSecs);  
            if (RandcpuUt[sizeRand] > 100) RandcpuUt[sizeRand] = 100;
            if (RandcpuUt[sizeRand] < 0.01) RandcpuUt[sizeRand] = 0;    
        }
        else
        {
            RandcpuUt[sizeRand] = 0;
        } 
        RandMsecs[sizeRand] = runSecs * 1000 / (double)blockReads;

        CloseHandle(hFile);
        useRand = TRUE;
    }

    if (testStarted) reView(blksizes, blksizes);   
    return TRUE;

} // readRAND


BOOL readDMA()
{
    HANDLE hFile;
    int j, k, c;
    BOOL  readIt;
    DWORD BytesIn;

    if (!useCache)
    {
        hFile = CreateFile(fileNames[0], GENERIC_READ,
                 FILE_SHARE_READ | FILE_SHARE_WRITE,
                 NULL, OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL
                 | FILE_FLAG_NO_BUFFERING
                 , NULL);
        if (hFile == INVALID_HANDLE_VALUE)
        {
           sprintf (fileMessage, " Cannot open file %s", fileNames[0]);
           return FALSE;
        }
        sprintf(windowText, " Rereading Block up to 100 times from %s at 1 to 1024 KBytes Per Block", fileNames[0]);
        SetWindowText(mainHWND, windowText);

        threadPass = 1;
        CalibrateTimer();

        readIt = ReadFile(hFile, &myData[0], DMAbytes, &BytesIn, NULL);
        if (measureCPU)
        {
            threadCount = 0;
            ResumeThread(hThreadHandle);    
            SetThreadPriority(hThreadHandle, THREAD_PRIORITY_IDLE);
        }
        getStartTime();
        c = 0;
        for (j=0; j<4; j++)
        {
            for (k=0; k<25; k++)
            {
                SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
                readIt = ReadFile(hFile, &myData[0], DMAbytes, &BytesIn, NULL);
            }
            getRunSecs();
            c = c + 25;
            if (runSecs > 1.0)
            {
                j = 4;
            }
        }
            
        if (runSecs == 0) runSecs = 0.00000001;   
        if (measureCPU)
        {
            SuspendThread(hThreadHandle);
            DMAcpuUt[sizeDMA] = 100.0 * ((runSecs - (double)threadCount / countsPerSec) / runSecs);  
            if (DMAcpuUt[sizeDMA] > 100) DMAcpuUt[sizeDMA] = 100;
            if (DMAcpuUt[sizeDMA] < 0.01) DMAcpuUt[sizeDMA] = 0;    
        }
        else
        {
            DMAcpuUt[sizeDMA] = 0;
        } 
        DMAMBPS[sizeDMA] = (double)(DMAbytes * c) / 1024 / 1024 / runSecs;
 
        CloseHandle(hFile);
        useDMA = TRUE;
    }
    if (testStarted) reView(blksizes, blksizes);   
    return TRUE;

} // readDMA

BOOL  readGFiles(int nw, int nr)
{   
    HANDLE hFile;
    BOOL  readIt;
    DWORD BytesIn;
    int  r;
    BOOL  chekR;
 
    if (useCache)
    {
        hFile = CreateFile(fileNames[useFile], GENERIC_READ,
             FILE_SHARE_READ | FILE_SHARE_WRITE,
             NULL, OPEN_EXISTING,
             FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
    }
    else
    {
        hFile = CreateFile(fileNames[useFile], GENERIC_READ,
             FILE_SHARE_READ | FILE_SHARE_WRITE,
             NULL, OPEN_EXISTING,
             FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN
                                | FILE_FLAG_NO_BUFFERING, NULL);
    }
    if (hFile == INVALID_HANDLE_VALUE)
    {
       sprintf (fileMessage, " Cannot open read file 2 %s", fileNames[useFile]);
       return FALSE;
    }
    sprintf(windowText, " Reading %s %d MBytes at %d Bytes Per Block", fileNames[useFile], grafmbytes, bytesPerBlock);
    SetWindowText(mainHWND, windowText);

    threadPass = 1;
    CalibrateTimer();

    if (measureCPU)
    {
        threadCount = 0;
        ResumeThread(hThreadHandle);    
        SetThreadPriority(hThreadHandle, THREAD_PRIORITY_IDLE);
    }
    chekR = 1;
    getStartTime();          
    for (r=0; r<numberBlocks; r++)
    {
        readIt = ReadFile(hFile, &myData[r * wordsPerBlock * bigOne], bytesPerBlock, &BytesIn, NULL);
        total = total | myData[r * wordsPerBlock * bigOne + wordsPerBlock - 1];
        if (readIt == 0) chekR = 0;                
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
           CloseHandle(hFile);
           sprintf (fileMessage, " File %s - Esc Pressed ", fileNames[useFile]);
           sprintf(windowText, " Reading %s %d MBytes ESCAPE Pressed", fileNames[useFile], grafmbytes);
           SetWindowText(mainHWND, windowText);
           if (measureCPU) SuspendThread(hThreadHandle);
           return FALSE;
        }
    }
        
    FlushFileBuffers(hFile);
    CloseHandle(hFile);
    if (chekR == 0 || BytesIn != bytesPerBlock)
    {
       sprintf (fileMessage,
          " File %s not read properly\n"
          " %d bytes read\n"
          " %d bytes expected\n"
          " %d serial blocks\n"
          " %d sumcheck",
           fileNames[useFile], BytesIn,
                      bytesPerBlock, numberBlocks, total);
       return FALSE;
    }
    
    
    getRunSecs();

    if (runSecs == 0) runSecs = 0.0000001;   

    if (measureCPU)
    {
        SuspendThread(hThreadHandle);
        cpuUt[useFile] = 100.0 * ((runSecs - (double)threadCount / countsPerSec) / runSecs);  
        if (cpuUt[useFile] > 100) cpuUt[useFile] = 100;
        if (cpuUt[useFile] < 0.01) cpuUt[useFile] = 0;    

    }
    else
    {
        cpuUt[useFile] = 0;
    } 

    GReadUt1[useSize][useFile] = cpuUt[useFile];
    GReadMBPS1[useSize][useFile] = (double)grafmbytes / runSecs;

    if (testStarted) reView(nw, nr);
    
    return TRUE;
    
} // readGFiles


BOOL saveResults()
{
     FILE *outfile;
     int i, j;
     double  r;

     SetCurrentDirectory(currentDir);
     outfile = fopen(logfile,"a+");
     if (outfile == NULL)
     {
         MessageBox(NULL, "Cannot open log file 1", logfile,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }
     fprintf (outfile, "\n\n ###########################################################\n");
     fprintf (outfile, "     Disk Test %s %s \n", version, timeday1);
     fprintf (outfile, "              Copyright Roy Longbottom 2001\n\n");
     fprintf(outfile, " %s\n", configData1);
     fprintf(outfile, " %s\n", configData2);
     fprintf(outfile, " %s\n", configData3);
     fprintf(outfile, "  %s %d MB, Free %d MB\n", diskPart, (int)totalDiskMB, (int)freeDiskMB );
     fprintf(outfile, "  Files %d Size %d MB Block Sizes %d ",
                               totalFiles, grafmbytes, maxBlockChoice+1-minBlockChoice);
     if(useCache)
     fprintf(outfile, "Use File Cache ");
     if(largeRAM)
     {
         fprintf(outfile, "RAM Data=File Size\n\n");
     }
     else
     {
         fprintf(outfile, "Minimum RAM Data\n"); 
     }
     if (strlen(note1) > 0 || strlen(note2) > 0 || strlen(note3) > 0)
     {
         fprintf(outfile, "  Notes: %s, %s, %s\n\n", note1, note2, note3);
     }
     else
     {
         fprintf(outfile, "\n");
     }
     fprintf (outfile, " Block   File ->\n");
     fprintf (outfile, "   KB"); 
     for (i=0; i<totalFiles; i++)
     {
         fprintf (outfile, "%5d ", i + 1);
     }
     fprintf (outfile, "\n");
     fprintf (outfile, "        Write MB/Second\n");

     for (i=minBlockChoice; i<maxBlockChoice+1; i++) 
     {
        fprintf (outfile, "%5d", gBlockSize[i]);
        for (j=0; j<totalFiles; j++)
        {
            if (GWriteMBPS1[i][j] > 1.0)
            {
               fprintf (outfile, "%6.1f", GWriteMBPS1[i][j]);
            }
            else if (GWriteMBPS1[i][j] > 0.1)
            {
               fprintf (outfile, "%6.2f", GWriteMBPS1[i][j]);
            }
            else 
            {
               fprintf (outfile, "%6.3f", GWriteMBPS1[i][j]);
            }
        }
        fprintf (outfile, "\n");
     }
     fprintf (outfile, "        Read  MB/Second\n");
    
     for (i=minBlockChoice; i<maxBlockChoice+1; i++)
     {
        fprintf (outfile, "%5d", gBlockSize[i]);
        for (j=0; j<totalFiles; j++)
        {
            if (GReadMBPS1[i][j] > 1.0)
            {
                fprintf (outfile, "%6.1f", GReadMBPS1[i][j]);
            }
            else if (GReadMBPS1[i][j] > 0.1)
            {
                fprintf (outfile, "%6.2f", GReadMBPS1[i][j]);
            }
            else
            {
                fprintf (outfile, "%6.3f", GReadMBPS1[i][j]);
            }                
        }
        fprintf (outfile, "\n");
     }
     fprintf (outfile, "        Write CPU Utilisation\n");
     
     for (i=minBlockChoice; i<maxBlockChoice+1; i++)
     {
        fprintf (outfile, "%5d", gBlockSize[i]);
        for (j=0; j<totalFiles; j++)
        {
            fprintf (outfile, "%6.1f", GWriteUt1[i][j]);
        }
        fprintf (outfile, "\n");
     }
     fprintf (outfile, "        Read  CPU Utilisation\n");
     
     for (i=minBlockChoice; i<maxBlockChoice+1; i++)
     {
        fprintf (outfile, "%5d", gBlockSize[i]);
        for (j=0; j<totalFiles; j++)
        {
            fprintf (outfile, "%6.1f", GReadUt1[i][j]);
        }
        fprintf (outfile, "\n");
     }
     if (useDMA)
     {
         fprintf (outfile, "\n DMA Reread block up to 100 times at 1 to 1024 Kbytes\n\n");
         fprintf (outfile, " Read  MB/Second\n");
         for (j=0; j<11; j++)
         {
            fprintf (outfile, "%6.1f", DMAMBPS[j]);
         }
         fprintf (outfile, "\n");
         fprintf (outfile, " Read  CPU Utilisation\n");
         for (j=0; j<11; j++)
         {
            fprintf (outfile, "%6.1f", DMAcpuUt[j]);
         }
         fprintf (outfile, "\n");
     }
     if (useRand)
     {
         fprintf (outfile, "\n Random Read 1KB out of 1, 2, 4, to file size Mbytes\n\n");
         fprintf (outfile, " Read  Milliseconds\n");
         for (j=0; j<11; j++)
         {
            r = RandMsecs[j];
            if (r > 999)
            {
                fprintf (outfile, "%6.0f", r);
            }
            else if (r < 1)
            {
                fprintf (outfile, "%6.3f", r);
            }
            else if (r < 10)
            {
                fprintf (outfile, "%6.2f", r);
            }
            else
            {                   
                fprintf (outfile, "%6.1f", r);
            }
         }
         fprintf (outfile, "\n");
         fprintf (outfile, " Read  CPU Utilisation\n");
         for (j=0; j<11; j++)
         {
            fprintf (outfile, "%6.1f", RandcpuUt[j]);
         }
         fprintf (outfile, "\n");
     }
     
     local_time();
     fprintf (outfile, "\n               End Disk Test %s\n", timeday);
     fflush (outfile);
     fclose(outfile);
     return TRUE;

} // saveResults 


BOOL saveErrors()
{
     FILE *outfile;

     SetCurrentDirectory(currentDir);
     outfile = fopen(logfile,"a+");
     if (outfile == NULL)
     {
         MessageBox(NULL, "Cannot open log file 2", logfile,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }
     fprintf (outfile, "\n\n ###########################################################\n");
     fprintf (outfile, " Disk Test %s %s \n", version, timeday);
     fprintf (outfile, "      Copyright Roy Longbottom 2001\n\n");
 
    fprintf (outfile, "\n\n ERROR %s\n\n", errorMsg);
    fflush (outfile);
    fclose(outfile);
    return TRUE;

} // saveErrors 


BOOL  saveBmp()
{

    HANDLE            hFile;
    LPBITMAPINFO      lpbmi = NULL;
    BITMAPFILEHEADER  bfh;
    int              dataLength;
    HDC               hdc = NULL;
    int              fileLength;
    DWORD             BytesWritten;
    char             *outBytes;
    BYTE              b;
    int              i, j, n;
    DWORD             g1, g2;

    PALETTEENTRY Vga16Palette[16] =
    {
        { 0,   0,   0,    0 },
        { 0x80,0,   0,    0 },
        { 0,   0x80,0,    0 },
        { 0x80,0x80,0,    0 },
        { 0,   0,   0x80, 0 },
        { 0x80,0,   0x80, 0 },
        { 0,   0x80,0x80, 0 },
        { 0x80,0x80,0x80, 0 },
        { 0xC0,0xC0,0xC0, 0 },
        { 0xFF,0,   0,    0 },
        { 0,   0xFF,0,    0 },
        { 0xFF,0xFF,0,    0 },
        { 0,   0,   0xFF, 0 },
        { 0xFF,0,   0xFF, 0 },
        { 0,   0xFF,0xFF, 0 },
        { 0xFF,0xFF,0xFF, 0 }
    };

    hdc = GetDC(mainHWND);

    // Allocate and fill out a BITMAPINFO struct for the new DIB

    dataLength = (recwin.right * recwin.bottom) / 2;

    outBytes = (char *) malloc(dataLength);
    if (outBytes == NULL)
    {
        strcpy(errorMsg, "Cannot allocate memory");
        if (autoRun)
        {
            saveErrors();
        }
        else
        {
             MessageBox(mainHWND, "Cannot allocate memory",
                 bmpfile, MB_ICONINFORMATION | MB_OK);
        }
        ReleaseDC(mainHWND, hdc);
        return FALSE;
    }

    lpbmi = (BITMAPINFO*) malloc( sizeof( BITMAPINFO ) + dataLength + 264 );
    if (lpbmi == NULL)
    {
        strcpy(errorMsg, "Cannot allocate memory");
        if (autoRun)
        {
            saveErrors();
        }
        else
        {
            MessageBox(mainHWND, "Cannot allocate memory",
               bmpfile, MB_ICONINFORMATION | MB_OK);
        }
        ReleaseDC(mainHWND, hdc);
        free(outBytes);
        return FALSE;
    }
    lpbmi->bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
    lpbmi->bmiHeader.biWidth = recwin.right;
    lpbmi->bmiHeader.biHeight = recwin.bottom;
    lpbmi->bmiHeader.biPlanes = 1;
    lpbmi->bmiHeader.biBitCount = 4;
    lpbmi->bmiHeader.biCompression = BI_RGB;
    lpbmi->bmiHeader.biSizeImage = dataLength;
    lpbmi->bmiHeader.biXPelsPerMeter = 0;
    lpbmi->bmiHeader.biYPelsPerMeter = 0;
    lpbmi->bmiHeader.biClrUsed = 0;
    lpbmi->bmiHeader.biClrImportant = 0;

    n = 0;
    
    for (i=0; i<recwin.bottom; i++)
    {
        for (j=0; j<recwin.right; j=j+2)
        {
            b = 0x00;
            g1 = GetPixel(hdc, j+0, recwin.bottom-1-i);
            g2 = GetPixel(hdc, j+1, recwin.bottom-1-i);
            
            if(g1 == RGB(255,0,0)) b = b | 0xC0;
            if(g2 == RGB(255,0,0)) b = b | 0x0C;
            if(g1 == RGB(0,0,255)) b = b | 0x90;
            if(g2 == RGB(0,0,255)) b = b | 0x09;
            if(g1 == RGB(255,255,255)) b = b | 0xF0;
            if(g2 == RGB(255,255,255)) b = b | 0x0F;
            outBytes[n] = b;
            n=n+1;
        }
    }

    memcpy(lpbmi->bmiColors, Vga16Palette, 64);    
    memcpy(lpbmi->bmiColors+64, outBytes, dataLength);    
    fileLength = 54 + 64 + dataLength;

    hFile = CreateFile(bmpfile, GENERIC_WRITE, 0, NULL,
                        CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
    {
         MessageBox(mainHWND, "Cannot open bmp file", bmpfile,
                         MB_ICONINFORMATION | MB_OK);
        ReleaseDC(mainHWND, hdc);
        free( lpbmi );
        free(outBytes);
        return FALSE;
    }
    
    // BMP file header

    bfh.bfType = 0x4d42;
    bfh.bfSize = fileLength;
    bfh.bfReserved1 = 0;
    bfh.bfReserved2 = 0;
    bfh.bfOffBits =  118;

    WriteFile( hFile, &bfh, 14, &BytesWritten, NULL );
    WriteFile( hFile, lpbmi, 104, &BytesWritten, NULL );
    WriteFile( hFile,  lpbmi->bmiColors+64, dataLength, &BytesWritten, NULL ); 

    CloseHandle( hFile ); 

    free( lpbmi );
    free(outBytes);
    ReleaseDC(mainHWND, hdc);
    
    return TRUE;
    
} // saveBmp()

 
BOOL quickRun()
{
    int i, size1, min1, max1, files1;

    size1 = grafmbytes;
    files1 = totalFiles;
    totalFiles = 2;
    useCache = FALSE;
    measureCPU = TRUE;
    largeRAM = FALSE;
    if (!startTests()) return FALSE;
    useSize = 6;
    grafmbytes = 4;
    grafSaved = FALSE;
    local_time();
    strcpy(timeday1, timeday);
    for (i=0; i<2; i++)
    {
        useFile = i;
        if (!write1File())
        {
            grafSaved = TRUE;
            grafmbytes = size1;
            endTests();
            return FALSE;
        }
    }
    for (i=0; i<2; i++)
    {
        useFile = i;
        if (!read1File())
        {
            grafSaved = TRUE;
            grafmbytes = size1;
            endTests();
            return FALSE;
        }
    }
    eraseFiles();
    min1 = minBlockChoice;
    max1 = maxBlockChoice;
    minBlockChoice = 6;
    maxBlockChoice = 6;
    endTests();                 
    minBlockChoice = min1;
    maxBlockChoice = max1;
    totalFiles = files1;
    grafSaved = TRUE;
    grafmbytes = size1;
    return TRUE;
      
} // quickRun


BOOL testFirst()
{
    HANDLE hFile;
    char  fileName[20];
    int   testData = 65536;
    int   testSize = 1024;
    int   theseRuns = 512;
    int   i, j, k;
    DWORD BytesOut;
    DWORD totalBytes;
    double runTime;
    HDC  hdc;
    int  thissize;
    char thisdata[200];
    char buf2[20];
    double ut1, ut2, sp1, sp2;
    SYSTEMTIME lpSystemTime;
    int  year;
    double  speedEst;
    BOOL     doMsg = FALSE;
    FILE *outfile;
    
    local_time();
    GetSystemTime(&lpSystemTime);
    year = lpSystemTime.wYear;

    SetCurrentDirectory(currentDir);
    outfile = fopen(logfile,"a+");
    if (outfile == NULL)
    {
        MessageBox(NULL, "Cannot open log file 3", logfile,
                                 MB_ICONWARNING | MB_OK);
        return FALSE;
    }
    fprintf (outfile, "\n\n ###########################################################\n");
    fprintf (outfile, " Calibrate Test %s %s \n", version, timeday);
    
    hdc = GetDC(mainHWND);
    SetBkColor(hdc, RGB(255,255,255));
    SetTextColor(hdc, RGB(0,0,0));
    GetClientRect(mainHWND, &recwin); 

    strcpy(fileName, "Test001.tst");

    myData = (int *)VirtualAlloc(NULL, testData, MEM_COMMIT, PAGE_READWRITE);
    if (myData == NULL)
    {
        strcpy(errorMsg, " VirtualAlloc - Cannot allocate memory");
        if (autoRun)
        {
            fprintf (outfile, "%s\n\n", errorMsg);
        }
        else
        {
            MessageBox(mainHWND, errorMsg, "VirtualAlloc",
                               MB_ICONWARNING | MB_OK);
        }
        ReleaseDC(mainHWND, hdc);
        fclose(outfile); 
        return FALSE;
    }
  
    for (i=0; i<3; i++)
    {
        sprintf(thisdata, "Checking Disk Speed at Block Size %d bytes, Pass %d of 3               ", testSize, i+1);
        thissize =  strlen(thisdata);
        ExtTextOut(hdc, 60, 20 + i * 20, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
        UpdateWindow(mainHWND);
                
        hFile = CreateFile(fileName, GENERIC_WRITE,
              FILE_SHARE_READ | FILE_SHARE_WRITE,
              NULL, CREATE_ALWAYS,
              FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING, NULL);
        if (hFile == INVALID_HANDLE_VALUE)
        {
           sprintf (fileMessage, " Cannot open write file %s", fileName);
           MessageBox(NULL, fileMessage, "Error", MB_ICONWARNING | MB_OK);
           VirtualFree(myData, 0, MEM_RELEASE);
           ReleaseDC(mainHWND, hdc);
           fclose(outfile); 
           return FALSE;
        }
        totalBytes = 0;
        filesWritten = TRUE;
        getStartTime();
        for (k=0; k<theseRuns; k++)
        {
            for (j=0; j<8; j++)
            {  
                WriteFile(hFile, &myData[0], testSize, &BytesOut, NULL);
                totalBytes = totalBytes + BytesOut;     
            }
            FlushFileBuffers(hFile);
            getRunSecs();
            if (runSecs > 1.0)
            {
                k = theseRuns + 1;
            }
        }
        CloseHandle(hFile);
    
        runTime = runSecs * 1048576.0 / (double)totalBytes;
        GWriteMBPS1[i][0] = runTime;
        if (runTime > 10)
        {
           minBlockChoice = minBlockChoice + 1;
        }
        testSize = testSize * 2;
        theseRuns = theseRuns / 2;       
        DeleteFile(fileName);
        filesWritten = FALSE;
    }

    VirtualFree(myData, 0, MEM_RELEASE);    
    if (runTime > 1.25)
    {
        totalFiles = (int)(50.0 / 8.0 / runTime);
        if (totalFiles < 2)
        {
            grafmbytes = 4;
            totalFiles = (int)(50.0 / 4.0 / runTime);
            if (totalFiles < 2)
            {
                grafmbytes = 2;
                totalFiles = (int)(50.0 / 2.0 / runTime);
                if (totalFiles < 2)
                {
                    grafmbytes = 1;
                    totalFiles = (int)(50.0 / runTime);
                    if (totalFiles < 1) totalFiles = 1;
                }
            }
        }
    } 
    strcpy(errorMsg, " Estimated MB/sec Write Tests 1, 2, 4 KB Blocks\n");
    for (i=0; i<3; i++)
    {
        sprintf(buf2, "%8.3f ", 1 / GWriteMBPS1[i][0]);
        strcat(errorMsg, buf2);
        GWriteMBPS1[i][0] = 0;
    }
//    MessageBox(mainHWND, errorMsg, "WARNING", MB_ICONWARNING | MB_OK);

    fprintf (outfile, "%s\n\n", errorMsg);
    if (minBlockChoice > 0 || totalFiles < 5 || grafmbytes < 8)
    {
        sprintf(errorMsg, " Disk appears to be slow. Default Parameters\n"
                          " changed - Min block size 1 KB to %d KB,\n"
                          " files 5 to %d, file size 8 MB to %d MB.\n"
                          " Click Variables button for different settings",
                   gBlockSize[minBlockChoice], totalFiles, grafmbytes);
        if (autoRun)
        {
            fprintf (outfile, "%s\n\n", errorMsg);
        }
        else
        {
            MessageBox(mainHWND, errorMsg, "WARNING", MB_ICONWARNING | MB_OK);
        }
    }

    sprintf(thisdata, "Checking CPU Utilisation and Read/Write Speed       ");
    thissize =  strlen(thisdata);
    ExtTextOut(hdc, 60, 90, ETO_CLIPPED, &recwin, thisdata, thissize, (LPINT) NULL);
    UpdateWindow(mainHWND);

    quickRun();
    
    ut1 = (GWriteUt1[6][0] + GWriteUt1[6][1]) / 2; 
    ut2 = (GReadUt1[6][0] + GReadUt1[6][1]) / 2;
    sp1 = (GWriteMBPS1[6][0] + GWriteMBPS1[6][1]) / 2;
    sp2 = (GReadMBPS1[6][0] + GReadMBPS1[6][1]) / 2;
    
    sprintf(errorMsg, " Write and Read two 4 MB files, 64 KB Block Size\n"
                      " Data Transfer Rate %6.2f and %6.2f MB/sec\n"
                      " Percent CPU Utilisation %6.2f and %6.2f\n\n"
                      , sp1, sp2, ut1, ut2);
    if (ut1 > 80 || ut2 > 80)
    {
        doMsg = TRUE;                  
        sprintf(thisdata, " CPU Utilisation High IS DMA OPTION SET?\n\n");
        strcat(errorMsg, thisdata);
    }
    else if (ut1 > 250 * sp1 / (double) megaHz || ut2 > 250 * sp2 / (double) megaHz)
    {
        doMsg = TRUE;
        sprintf(thisdata, " CPU Utilisation Might Be High for %d MHz CPU\n Check Results Via Main Tests\n\n", megaHz);
        strcat(errorMsg, thisdata);
    }    

    if (sp1 * 1.4 < sp2)
    {
        doMsg = TRUE;
        sprintf(thisdata, " Writing speed is much slower than reading speed\n\n");
        strcat(errorMsg, thisdata);
    }
    if (year < 2001) year = 2001;
    speedEst = pow(1.28, (double)(year - 2001)) * 20;
    if (sp2 < speedEst)
    {
        sprintf(thisdata, " Reading Speed Slow For Year %d?\n", year);
        strcat(errorMsg, thisdata);
        sprintf(thisdata, " Is disk data fragmented? - Defragment before running?\n\n");
        strcat(errorMsg, thisdata);
        doMsg = TRUE;
    }
    fprintf (outfile, "%s\n", errorMsg);
    if (doMsg)
    {
        if (!autoRun)
        {
            MessageBox(mainHWND, errorMsg, "WARNING", MB_ICONWARNING | MB_OK);
        }
    }
    GWriteMBPS1[6][0] = 0;
    GReadMBPS1[6][0]  = 0;
    GWriteUt1[6][0] = 0;
    GReadUt1[6][0]  = 0;
    GWriteMBPS1[6][1] = 0;
    GReadMBPS1[6][1]  = 0;
    GWriteUt1[6][1] = 0;
    GReadUt1[6][1]  = 0;
    if (autoRun)
    {
        strcpy(errorMsg, " Command ");
        strncat(errorMsg, commandLine, 200);
        fprintf (outfile, "%s\n\n", errorMsg);
    }
    if (commandError)
    {
        strcpy(errorMsg, " Command Error ");
        strncat(errorMsg, commandLine, 200);
        sprintf (thisdata, "%s\n\n", errorMsg);
        MessageBox(mainHWND, errorMsg, "WARNING", MB_ICONWARNING | MB_OK);
    }

    ReleaseDC(mainHWND, hdc); 
    fflush (outfile);
    fclose(outfile);
    return TRUE;
    
} // testFirst

static BOOL GetFileName()
{
    static char    filterList1[] = "File (*.txt)" \
                                        "\0" \
                                        "*.txt" \
                                        "\0\0";
    static char    filterList2[] = "File (*.bmp)" \
                                        "\0" \
                                        "*.bmp" \
                                        "\0\0";
    OPENFILENAME        of;
    int                 rc;
    
    if (loadIt)
    {
        grafFile[ 0 ] = 0;
    }
    else
    {
        strcpy(grafFile, bmpfile);
    }
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = mainHWND;
    of.lpstrFilter = (LPSTR) filterList1;
    if (!loadIt) of.lpstrFilter = (LPSTR) filterList2;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = grafFile;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.Flags = OFN_HIDEREADONLY;

    if (loadIt)
    {
        rc = GetOpenFileName( &of );
    }
    else
    {
        rc = GetSaveFileName( &of );
        strcpy(bmpfile, of.lpstrFile);
    }
   
    return( rc );
    
} // GetFileName

BOOL loadFile()
{
     FILE *infile;
     char buf[350];
     char buf2[350];
     char buf3[350];
     BOOL fin = FALSE;
     BOOL skip;
     int  gotGraf = 0;
     int  i, j, i1, i2, i3;
     int  sizes;
     int  ccount[200];
     char fpnumber[6];
     double utsum;
     int  mbYNC;
     int  grafCount = 0;
     int  count2 = 0;

     fromLog = FALSE;
     viewDMA = FALSE;
     viewRand = FALSE;


     infile = fopen(fileToRead,"r");
     if (infile == NULL)
     {
         MessageBox(NULL, "Cannot open graph log file 4", fileToRead,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }
     while (!fin && fgets(buf, 120, infile) != NULL)
     {
        if (strnicmp(buf, "     Disk test DiskGraf", 23) == 0)
        {
            grafCount = grafCount + 1;
        }
     }
     fclose(infile);

     infile = fopen(fileToRead,"r");
     if (infile == NULL)
     {
         MessageBox(NULL, "Cannot open graph log file 5", logfile,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }

    while (!fin && fgets(buf, 120, infile) != NULL)
    {
        skip = FALSE;
        if (strnicmp(buf, "     Disk test DiskGraf", 23) == 0)
        {
         if (strnicmp(buf, "     Disk test DiskGraf FAT16 V ", 32) == 0)
            {
                sscanf(buf, "%32c%s", buf2, buf3);
                sprintf(versionIn, "FAT16 V %s", buf3);
            }
            else
            {
                sscanf(buf, "%32c%s", buf2, buf3);
                sprintf(versionIn, "Version %s", buf3);
            }
            count2 = count2 + 1;
            for (i=0; i<200; i++)
            {
                 ccount[i] = 0;
            }
            for (i=0; i<blksizes; i++)
            {
                for (j=0; j<maxfiles; j++)
                {
                    GWriteMBPS1[i][j] = 0;
                    GReadMBPS1[i][j]  = 0;
                    GWriteUt1[i][j] = 0;
                    GReadUt1[i][j]  = 0;
                }
            }
            for (i=0; i<11; i++)
            {
                 DMAcpuUt[i] = 0;
                 DMAMBPS[i] = 0;                 
                 RandcpuUt[i] = 0;
                 RandMsecs[i] = 0;                 
            }
            useDMA = FALSE;
            useRand = FALSE;
            sscanf(buf, "%36c%25c ", buf2, timeday);
            fgets(buf, 120, infile);
            fgets(buf, 120, infile);
            fgets(buf, 120, infile);
            fgets(buf, 120, infile);
            strncpy(configData1, buf, strlen(buf) - 1);
            configData1[strlen(buf)] = 0;
            fgets(buf, 120, infile);
            strncpy(configData2, buf, strlen(buf) - 1);
            configData2[strlen(buf)] = 0;
            fgets(buf, 120, infile);
            strncpy(configData3, buf, strlen(buf) - 1);
            configData3[strlen(buf)] = 0;
            fscanf(infile, "%s %d %s %s %d %s", diskPart, &i1, buf, buf, &i2, buf);
            totalDiskMB = (double)i1;
            displayOnly = TRUE;
            freeDiskMB = (double)i2;
            fscanf(infile, "%s %d %s %d %15c %d", buf, &totalFiles, buf, &grafmbytes, buf, &sizes);
            fgets(buf, 16, infile);
            useCache = FALSE;
            if (strnicmp(buf, " Use", 4) == 0)
            {
                useCache = TRUE;
                fgets(buf, 16, infile);
            }
            largeRAM = TRUE;
            if (strnicmp(buf, " Min", 4) == 0) largeRAM = FALSE;
            fgets(buf, 120, infile);
            strcpy(note1, "");
            strcpy(note2, "");
            strcpy(note3, "");
            fgets(buf3, 10, infile);   // Notes: ?
            if (strlen(buf3) > 7)
            {
                fgets(buf2, 120, infile);
                i1 = 0;
                for (i=0; i<lstrlen(buf2); i++)
                {
                    if (buf2[i] == 44  && buf2[i+1] == 32)
                    {
                        ccount[i1] = i;
                        i1 = i1 + 1;
                    }
                }
                if (i1 > 2)
                {
                    i2 = ccount[0];
                    i3 = ccount[1];
                    for (i=0; i<i1; i++)
                    {
                        if (ccount[i] < 19) i2 = ccount[i];
                        if (ccount[i] < 38) i3 = ccount[i];
                    }
                    ccount[0] = i2;
                    ccount[1] = i3;
                }
                i1 = 0;
                if (ccount[0] < 100)
                {
                    for (i=0; i<ccount[0]; i++)
                    {
                        note1[i1] = buf2[i];
                        i1 = i1 + 1;
                    }
                    note1[i1] = 0;
                }
                i1=0;
                if (ccount[1] - ccount[0] < 100)
                {
                    for (i=ccount[0]+2; i<ccount[1]; i++)
                    {
                        note2[i1] = buf2[i];
                        i1=i1+1;
                    }
                    note2[i1] = 0;
                }
                i1=0;
                if (strlen(buf2) - ccount[1] < 100)
                {
                    for (i=ccount[1]+2; i<lstrlen(buf2)-1; i++)
                    {
                        note3[i1] = buf2[i];
                        i1=i1+1;
                    }
                    note3[i1] = 0;
                }
                fgets(buf, 120, infile);
            }
            fgets(buf, 120, infile);
            if (strnicmp(buf, " Block", 6) != 0) skip = TRUE;
            if (!skip)
            {
                fgets(buf, 120, infile);
                fgets(buf, 120, infile);
                if (strnicmp(buf, "        Write", 13) != 0) skip = TRUE;
            }
            if (!skip)
            {
                fscanf(infile, "%5d", &i1);

                for (i=0; i<blksizes; i++)
                {
                    if (i1 == gBlockSize[i])
                    {
                        minBlockChoice = i;
                        maxBlockChoice = i + sizes - 1;
                    }
                }
                for (i=minBlockChoice; i<maxBlockChoice+1; i++)
                {
                    if (i > minBlockChoice)
                    {
                        fscanf(infile, "%5d", &i2);
                    }
                    for (j=0; j<totalFiles; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        GWriteMBPS1[i][j] = strtod(fpnumber, NULL);
                    }
                    fgets(buf, 120, infile);  // CR
                }

                fgets(buf, 120, infile);
                if (strnicmp(buf, "        Read", 12) != 0) skip = TRUE;
            }
            if (!skip)
            {                
                for (i=minBlockChoice; i<maxBlockChoice+1; i++)
                {
                    fscanf(infile, "%5d", &i2);
                    for (j=0; j<totalFiles; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        GReadMBPS1[i][j] = strtod(fpnumber, NULL);
                    }
                    fgets(buf, 120, infile);  // CR
                }

                utsum = 0;
                fgets(buf, 120, infile);
                if (strnicmp(buf, "        Write CPU Ut", 20) != 0) skip = TRUE;
            }
            if (!skip)
            {                
                for (i=minBlockChoice; i<maxBlockChoice+1; i++)
                {
                    fscanf(infile, "%5d", &i2);
                    for (j=0; j<totalFiles; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        GWriteUt1[i][j] = strtod(fpnumber, NULL);
                        utsum = utsum + GWriteUt1[i][j];
                    }
                    fgets(buf, 120, infile);  // CR
                }

                fgets(buf, 120, infile);
                if (strnicmp(buf, "        Read  CPU Ut", 20) != 0) skip = TRUE;
            }
            if (!skip)
            {                
                for (i=minBlockChoice; i<maxBlockChoice+1; i++)
                {
                    fscanf(infile, "%5d", &i2);
                    for (j=0; j<totalFiles; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        GReadUt1[i][j] = strtod(fpnumber, NULL);
                        utsum = utsum + GReadUt1[i][j];
                    }
                    fgets(buf, 120, infile);  // CR
                }
                measureCPU = TRUE;
                fromLog = TRUE;
                if (utsum == 0) measureCPU = FALSE;                

                fgets(buf, 120, infile);
                fgets(buf, 120, infile);
                if (strnicmp(buf, " DMA", 4) == 0) useDMA = TRUE;
                if (useDMA)
                {
                    fgets(buf, 120, infile);
                    fgets(buf, 120, infile);
                    for (j=0; j<11; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        DMAMBPS[j] = strtod(fpnumber, NULL);
                    }
                    fgets(buf, 120, infile);  // CR
                    fgets(buf, 120, infile);
                    for (j=0; j<11; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        DMAcpuUt[j] = strtod(fpnumber, NULL);
                    }
                    fgets(buf, 120, infile);  // CR
                }
                fgets(buf, 120, infile);
                fgets(buf, 120, infile);
                if (strnicmp(buf, " Rand", 5) == 0) useRand = TRUE;
                if (useRand)
                {
                    fgets(buf, 120, infile);
                    fgets(buf, 120, infile);
                    for (j=0; j<11; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        RandMsecs[j] = strtod(fpnumber, NULL);
                    }
                    fgets(buf, 120, infile);  // CR
                    fgets(buf, 120, infile);
                    for (j=0; j<11; j++)
                    {
                        fscanf(infile, "%6c", fpnumber);
                        RandcpuUt[j] = strtod(fpnumber, NULL);
                    }
                    fgets(buf, 120, infile);  // CR
                }
                reView(blksizes, blksizes);
                sprintf(fileMessage, "Search for another graph?");
                sprintf(buf3, " Loaded graph %d of %d from file %s", count2, grafCount, fileToRead);
                if (count2 < grafCount)
                {
                    mbYNC = MessageBox(mainHWND, fileMessage, buf3,
                                        MB_ICONQUESTION | MB_YESNO);
                    if (mbYNC == IDNO)
                    {
                       fin = TRUE;
                       gotGraf = 1;
                    }
                    else
                    {
                        gotGraf = 2;
                    }
                }
                else
                {
                    fin = TRUE;
                    gotGraf = 1;
                }
            }
        }
    }
    sprintf(buf3, " Loading graph from file %s", fileToRead);
    if (gotGraf == 0)
    {
        MessageBox(NULL, "No graph found", buf3, MB_ICONWARNING | MB_OK);
    }
    else if (gotGraf == 2)
    {
        MessageBox(NULL, "No more graphs found", buf3, MB_ICONWARNING | MB_OK);
    }    
    fclose(infile);
    return TRUE;
     
} // loadFile()


BOOL  startTests()
{
    int i, j;
    DWORD di;
    SetCursorPos(600, 440);
    char newStart[4] = "  ";
    
    grafSaved = FALSE;
    local_time();
    strcpy(timeday1, timeday);
    WhatConfig();
    GetDiskStats();
    displayOnly = FALSE;
    keyPress = GetAsyncKeyState(VK_ESCAPE);

    if (strlen(currentDir) < 2)
    {
        strcpy (newStart, currentDir);
    }
    else
    {
        strncpy(newStart, currentDir, 2);
    }
    sprintf(diskPart, "%sDisk/partition", newStart); 
    
    for (i=0; i<blksizes; i++)
    {
        for (j=0; j<maxfiles; j++)
        {
            GWriteMBPS1[i][j] = 0;
            GReadMBPS1[i][j]  = 0;
            GWriteUt1[i][j] = 0;
            GReadUt1[i][j]  = 0;
        }
    }
    for (i=0; i<11; i++)
    {
         DMAcpuUt[i] = 0;
         DMAMBPS[i] = 0;
         RandcpuUt[i] = 0;
         RandMsecs[i] = 0;
    }
 
    if (testStarted) reView(blksizes, blksizes);

    bytesPerfile = grafmbytes * 1024 * 1024;
    if (largeRAM)
    {
        arrayBytes = bytesPerfile;
        bigOne = 1;
    }
    else
    {
        arrayBytes = maxBytesPerBlock;
        bigOne = 0;
    }
    arrayWords = arrayBytes / 4;
    myData = (int *)VirtualAlloc(NULL, arrayBytes, MEM_COMMIT, PAGE_READWRITE);
    if (myData == NULL)
    {
        strcpy(errorMsg, "Cannot allocate memory");
        if (autoRun)
        {
            saveErrors();
        }
        else
        {
            MessageBox(mainHWND, errorMsg, "VirtualAlloc",
                               MB_ICONWARNING | MB_OK);
        }
        return FALSE;
    }
    readWrite = TRUE;
    if (testStarted) reView(blksizes, blksizes);

    sprintf(windowText, " Filling Memory With Data");
    SetWindowText(mainHWND, windowText);

    for ( di = 0; di < arrayWords; di++)
    {
         myData[di] = di+1;
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf (fileMessage, " Creating Data - Esc Pressed ");
             sprintf(windowText, " Ceating Data ESCAPE Pressed");
             SetWindowText(mainHWND, windowText);                                         
             if (autoRun)
             {
                 saveErrors();
             }
             else
             {
                 MessageBox(mainHWND, fileMessage, "Escape",
                                MB_ICONWARNING | MB_OK);
             }
             grafSaved = TRUE;
             endTests();
             return FALSE;
         }
    }

    return TRUE;

} // startTests()



BOOL  endTests()
{
    VirtualFree(myData, 0, MEM_RELEASE);
    readWrite = FALSE;
    if (testStarted) reView(blksizes, blksizes);
    if (!grafSaved)
    {
        if (testStarted) saveResults();
    }
    grafSaved = TRUE;
    SetWindowText(mainHWND, TITLE);
    SetCursor(LoadCursor(NULL, IDC_ARROW));

    return TRUE;
}

BOOL eraseFiles()
{
    int j;
    
    for (j=0; j<totalFiles; j++)
    {
        DeleteFile(fileNames[j]);
    }
    filesWritten = FALSE;
    GetDiskStats();
    return TRUE;
}

 // Assembly routine

//  *************** CPU Soak **************

void readme()
{
    __asm
    {
      mov     edx, 1
      mov     ecx, -1
      mov     ebx, 0
      mov     eax, 500
  lp: add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      add     ebx, ecx
      add     ebx, edx
      dec     eax     
      jnz     lp                   
    }
}
